package clarion.acs;
import clarion.tools.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * this class is to implement the basic concept of "Value Competition Grouping" ,
 * new in CLARION mdoel.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public int decideCandidate(): Decides the candidate. used in BL operations.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements the concept of "Value Competition Grouping" , new in
 * CLARION mdoel. It makes easy implementation of multiple-values in one dimension.
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>AcsAction</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class VCG {

  /** the number of elements. */
  protected int elementNum;
  /** the candidate. */
  protected int candidate;
  /** the activations. */
  protected double[] activations;

  /** constructor.
   *  @param elementNum the number of elements. */
  public VCG( int elementNum )
  {
     this.elementNum = elementNum;
     activations = new double[ elementNum ];
  }

  /** Returns the number of elements.
   *  @return the number of elements. */
  public int getElementNum()
  {
     return elementNum;
  }

  /** Returns the candidate.
   *  @return the candidate. */
  public int getCandidate()
  {
     return candidate;
  }

  /** Returns the activations.
   *  @return the activations. */
  public double[] getActivations()
  {
     return activations;
  }

 /** Returns the MAX activation in this VCG.
  *  @return the MAX activation in this VCG. */
  public double getMaxActivation()
  {
     int maxIdx = CommonMethods.getMaxValueIdx( activations );
     return activations[maxIdx];
  }

  /** Sets the candidate.
   *  @param candidate the candidate to set. */
  public void setCandidate( int candidate )
  {
     this.candidate = candidate;
  }

  /** Sets the activations.
   *  @param activations the activations to set. */
  public void setActivations( double[] activations )
  {
     for( int i = 0; i < activations.length; i++ )
          this.activations[i] = activations[i];
  }

  /** Sets the activations.
   *  @param activeValue the active value (candidate) to set. */
  public void setActivations( int activeValue )
  {
     this.setCandidate( activeValue );
     for( int i = 0; i < activations.length; i++ )
          activations[i] = 0.0;
     activations[ activeValue ] = 1.0;
  }

  /** Decides the candidate. used in BL operations.
   *  @return the candidate. */
  public int decideCandidate()
  {
     StochasticDecider decider = new StochasticDecider( elementNum, 0.1 );
     candidate = decider.getRecommendation( activations );
     return candidate;
  }
}