package clarion.acs.net;
import clarion.system.*;
import clarion.common.*;
import clarion.tools.*;
import clarion.acs.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is the base class to represent a knowledge type in an ACS network.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> abstract public Chunk[] getAllConclusions(): get all conclusions drawn by
 * the component.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class extends the base class <em>GeneralNetComp</em> and is the base class
 * for knowledge representation in an ACS action decision network.
 * <li> Usage of this class. <br>
 * It is instantiated inside an ACS action decision network object: <em>AcsNet</em>.
 * </ul>
 * @author Xi Zhang
 *
 */

public abstract class AcsNetComp extends GeneralNetComp {

  /** the network type : EX, GS or WM. */
  protected int netType;
  /** the length of NACS-control outputs. */
  protected int CONTROL_OUTPUT_NUM;
  /** the # of NACS-control dims. */
  protected int CONTROL_OUTPUT_DIM_NUM;
  /** the dim-val info of NACS-control outputs. */
  protected short[] controlOutputDVs;
  /** the start offsets of each NACS-control dim in an one-dimenisonal array. */
  protected short[] controlOutputOffsets;
  /** the length of NACS-control outputs. */
  protected int FULL_CONTROL_OUTPUT_NUM;
  /** the # of NACS-control dims. */
  protected int FULL_CONTROL_OUTPUT_DIM_NUM;
  /** the dim-val info of NACS-control outputs. */
  protected short[] fullControlOutputDVs;
  /** the start offsets of each NACS-control dim in an one-dimenisonal array. */
  protected short[] fullControlOutputOffsets;
 /** current fully suggested action by <tt>this</tt> network component.
   *  the first dim: ACTION_NORMAL or ACTION_CONTROL.
   *  the second dim: the relevant output dims specified by this net comp.
   *  Here, the output dims is the full output dims.
   *  the third dim: the active values in one dim. */
  protected short[][][] fullSuggestedAction;
  /** used to manage action. */
  protected AcsAction acsAction;
  /** used to inialize acsAction.
   *  @see clarion.acs.AcsAction */
  protected short[] outputFormatDimNums, outputFormatDimValNums[];
  /** returns all of the conclusions drawn by this component.
   *  @return the conclusions drawn by this component. */
  abstract public Chunk[] getAllConclusions();
}