package clarion.acs.net.auxiliary;
import clarion.system.*;
import clarion.common.*;

/** this class is used for recording the usage of an action chunk. */
public class ActionUsageInfo {

   /** maximal number of usage. */
   protected static final int MAX_USAGE_NUM = 1000;
   /** the number of usage so far. */
   protected int usageNum;
   /** the array of usages. */
   protected int[] usages = new int[ MAX_USAGE_NUM ];
   /** parameters for BLA. */
   protected  double c, d, iBLA;
   /** BLA. */
   protected double BLA;

   /** constructor.
    *  @param initTime the initial time for setting up this action chunk.
    *  @param params the values of the parameters for BLA. */
   public ActionUsageInfo( int initTime, double[] params )
   {
      usageNum = 1;
      usages[0] = initTime;
      c = params[Global.BLA_C];
      d = params[Global.BLA_D];
      iBLA = params[Global.BLA_I];
   }

   /** Returns the number of usage.
    *  @return the number of usage. */
   public int getUsageNum()
   {
      return usageNum;
   }

   /** Returns the BLA.
    *  @return the BLA. */
   public double getBla()
   {
      return BLA;
   }

   /** increase the number of usage.
    *  @param currentTime current time. */
   public void increaseUsage( int currentTime )
   {
      if( currentTime <= usages[usageNum-1] )
          return;
      usages[usageNum++] = currentTime;
   }

   /** Calculates the BLA with current time.
    *  @param curTime current time. */
   public void calcBla( int curTime )
   {
       double sum = iBLA;
       for( int i = 0; i < usageNum; i++ ) {
            int timeLaps = curTime - usages[ i ];
            if( timeLaps <= 0 ){
                //System.out.println( "error usage recording in RuleAttributes.java" );
                break;
            }
            sum += c * Math.pow( (double)timeLaps, -d );
       }
       BLA = sum;
       if( BLA < 0.01 )
           BLA = 0.01;
   }
}