package clarion.acs.net.auxiliary;
import java.util.BitSet;
import clarion.system.*;

/** This class implements the key to elements in a collection which stores huge
 *  number of action chunks. */
public class KeyToValue implements Comparable {
   protected int size;
   protected BitSet keys;

   public KeyToValue( int size0, int size1, short[][][] action, short[] offsets0, short[] offsets1 )
   {
      size = size0 + size1;
      keys = new BitSet( size );
      for( int i = 0; i < action[Global.ACTION_NORMAL].length; i++ ){
           if( action[Global.ACTION_NORMAL][i] == null )
               continue;
           for( int j = 0; j < action[Global.ACTION_NORMAL][i].length; j++ ) {
                int bitIndex = offsets0[i] + action[Global.ACTION_NORMAL][i][j];
                keys.set( bitIndex );
           }
      }
      if( action[Global.ACTION_CONTROL] != null ){
          for( int i = 0; i < action[Global.ACTION_CONTROL].length; i++ ){
               if( action[Global.ACTION_CONTROL][i] == null )
                   continue;
               for( int j = 0; j < action[Global.ACTION_CONTROL][i].length; j++ ) {
                    int bitIndex = size0 + offsets1[i] + action[Global.ACTION_CONTROL][i][j];
                    keys.set( bitIndex );
               }
          }
      }
   }

   public int getSize()
   {
      return size;
   }

   public BitSet getKeys()
   {
      return keys;
   }

   public int hashCode()
   {
      int code = 0;
      int length = Math.min( size, 32 );
      for( int i = 0; i < length; i++ )
           code = code << 1 + ((keys.get(i))?1:0);
      return code;
   }

   public boolean equals( Object obj )
   {
      if( obj == null || !( obj instanceof KeyToValue ) || getSize() != ((KeyToValue)obj).getSize() )
          return false;
      BitSet otherKeys = ((KeyToValue)obj).getKeys();
      return keys.equals( otherKeys );
   }

   public int compareTo(Object obj) throws ClassCastException
   {
      KeyToValue otherKeyValue = null;
      try{
         otherKeyValue = (KeyToValue)obj;
      }
      catch( ClassCastException e ){
             System.out.println( e );
      }
      BitSet otherKeys = otherKeyValue.getKeys();
      int length = Math.min( keys.length(), otherKeys.length() );
      for( int i = 0; i< length; i++ )
           if( keys.get(i) != otherKeys.get(i) )
               return keys.get(i)?1:-1;
      return 0;
   }
}
