package clarion.nacs;
import clarion.system.*;
import clarion.common.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the samples in Abstract Episodic Memory. In this class,
 * each sample contains six parts: current state, current action, next state,
 * reinforcement, state distribution and potential next states.
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements samples in AEM and it makes easier the implementation of
 * training of IDN using these decomposed samples.
 * <li> Usage of this class. <br>
 * It is instantiated inside <em>AbsEM</em> object.
 * </ul>
 * @author Xi Zhang
 */

public class AbsEMSample{

  /** current state. */
   Chunk  curState;
   /** current action. */
   Chunk  curAction;
   /** next state. */
   Chunk  nextState;
   /** currently received reinforcement. */
   double  reward;
   /** state distributions. */
   double stateDistributions[];
   /** potential next states. */
   Chunk  potentialNextStates[];

  /** constructor.
    *  @param curState the current state part.
    *  @param curAction the current attion part.
    *  @param nextState the next state part.
    *  @param reward the reinforcement part.
    *  @param stateDistributions the state distributions.
       @param potentialNextStates the potential next states. */
   public AbsEMSample( Chunk curState, Chunk curAction, Chunk nextState, double rewardIdx,
                       double[] stateDistributions, Chunk[] potentialNextStates )
   {
      this.curState = curState;
      this.curAction = curAction;
      this.nextState = nextState;
      this.reward = reward;
      this.stateDistributions = stateDistributions;
      this.potentialNextStates = potentialNextStates;
   }

   /** Returns the part of current state.
    *  @return the part of current state. */
   public Chunk getCurState()
   {
      return curState;
   }
   /** Returns the part of current action.
    *  @return the part of current action. */
   public Chunk getCurAction()
   {
      return curAction;
   }

   /** Returns the part of the next state.
    *  @return the part of the next state. */
   public Chunk getNextState()
   {
      return nextState;
   }

   /** Returns the part of the reinforcement.
    *  @return the part of the reinforcement. */
   public double getReward()
   {
      return reward;
   }

   /** Returns the part of state distributions.
    *  @return the part of state distributions. */
   public double[] getStateDistributions()
   {
      return stateDistributions;
   }

   /** Returns the part of potential next states.
    *  @return the part of potential next states. */
   public Chunk[] getPotentialNextStates()
   {
      return potentialNextStates;
   }
}