package clarion.nacs;
import clarion.system.*;
import clarion.common.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the ACS samples in Episodic Memory. In this class,
 * each ACS sample contains four parts: current state, current action, next state
 * and reinforcement.
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class implements ACS samples in EM and it makes easier the implementation of
 * training of IDN or AEM using these decomposed samples.
 * <li> Usage of this class. <br>
 * It is instantiated inside <em>EM</em> object.
 * </ul>
 * @author Xi Zhang
 */

public class EMAcsSample{

   /** current state. */
   Chunk  curState;
   /** current action. */
   Chunk  curAction;
   /** next state. */
   Chunk  nextState;
   /** currently received reinforcement. */
   double  reward;

   /** constructor.
    *  @param curState the current state part.
    *  @param curAction the current attion part.
    *  @param nextState the next state part.
    *  @param reward the reinforcement part. */
   public EMAcsSample( Chunk curState, Chunk curAction, Chunk nextState, double reward )
   {
      this.curState = curState;
      this.curAction = curAction;
      this.nextState = nextState;
      this.reward = reward;
   }

   /** Returns the part of current state.
    *  @return the part of current state. */
   public Chunk getCurState()
   {
      return curState;
   }

   /** Returns the part of current action.
    *  @return the part of current action. */
   public Chunk getCurAction()
   {
      return curAction;
   }

   /** Returns the part of the next state.
    *  @return the part of the next state. */
   public Chunk getNextState()
   {
      return nextState;
   }

   /** Returns the part of the reinforcement.
    *  @return the part of the reinforcement. */
   public double getReward()
   {
      return reward;
   }

   /** methods from class <em>Object</em> */
   public String toString()
   {
      String str = "cs="+ curState + "act=" + curAction + ", ns=" + nextState + "rwd=" + reward;
      return str;
   }
}