package clarion.options;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import clarion.system.*;
import clarion.options.dims.*;
import clarion.options.acs.*;
import clarion.options.nacs.*;
import clarion.options.supervise.*;
import clarion.options.rt.*;
import clarion.options.inter.*;
import clarion.options.taskspec.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the initial screen of configuring CLARION model by GUI.
 * It extends the base class <em>GuiTemplate</em>.  It contains the following
 * configuration entries: Subsystem selection, Input and Output dimensions,
 * Action-Centered Subsystem, Non-Action-Centered Subsystem, Supervisory
 * Subsystems and Response Time Determination.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void loadSubSys(): Loads and initializes the CLARION subsystems.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the CLARION model by GUI
*  (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>NewSimulator</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiClarion extends GuiTemplate{

  final int BTN_SUBSYS = 0, BTN_DIM = 1, BTN_ACS = 2, BTN_NACS = 3;
  final int BTN_SUPER = 4, BTN_RT = 5;
  String buttonTexts[] = {
     "Subsystem selection",
     "Input and Output dimensions",
     "Action-Centered Subsystem  Options",
     "Non-Action-Centered Subsystem Option",
     "Supervisory Subsystems Option",
     "Response Time Determination Option",
  };
  /** configuration of subsystems of CLARION by GUI. */
  GuiTemplate[] guiSubSys;
  /** other configurations. */
  SelectionTemplate[] selOptions;

  /** constructor.
   *  @param num the number of components needing configuration.
   *  @param globals the global settings with the running agents. */
  public GuiClarion( int num, Global[] globals )
  {
     super( num, "CLARION PARAMETER SETTINGS", globals );
     setButtonTexts( buttonTexts );
  }

  /** Loads and initializes the CLARION subsystems. */
  public void loadSubSys()
  {
     manager.load( Global.CLARION_OPTIONS );
     guiSubSys = new GuiTemplate[ 4 ];
     guiSubSys[ 0 ] = new GuiDims( 3 );
     guiSubSys[ 1 ] = new GuiACS( 4 );
     guiSubSys[ 2 ] = new GuiNACS( 5 );
     guiSubSys[ 3 ] = new GuiSupervisory( 3 );
     for( int i = 0; i < 4; i++ )
          guiSubSys[i].init( this );
     selOptions = new SelectionTemplate[ 4 ];
     selOptions[ 0 ] = new SelectionSubSys( this, 400, 200, manager );
     selOptions[ 1 ] = new SelectionRT( this, 660, 900, manager );
     // update button status.
     optButtons[BTN_NACS].setEnabled( manager.isNacsOn() );
     optButtons[BTN_SUPER].setEnabled( manager.isSupervisoryOn() );
     optButtons[BTN_RT].setEnabled( manager.isRtOn() );
  }

  /** the routine of processing the changes when returning from other GUIs. */
  public void returnFrom()
  {
     if( activeButtonIdx == 0 ){
         optButtons[BTN_NACS].setEnabled( manager.isNacsOn() );
         optButtons[BTN_SUPER].setEnabled( manager.isSupervisoryOn() );
         optButtons[BTN_RT].setEnabled( manager.isRtOn() );
     }
  }

  /** Adds action listeners to the option buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int focus = i;
          final int idx;
          switch( i ){
            case 1:
            case 2:
            case 3:
            case 4:
                  idx = i-1;
                  optButtons[i].addActionListener(  new  ActionListener() {
                             public void actionPerformed( ActionEvent e ){
                                activeButtonIdx = focus;
                                guiSubSys[ idx ].setEnabled( true );
                                guiSubSys[ idx ].setVisible( true );
                                mainFrame.setEnabled( false );
                                mainFrame.setVisible( false );
                             }
                         } );
                   break;
            case 0:
            case 5:
                  idx = (i==0) ? i : i-4;
                  optButtons[i].addActionListener( new  ActionListener() {
                             public void actionPerformed( ActionEvent e ){
                                activeButtonIdx = focus;
                                selOptions[ idx ].setEnabled( true );
                                selOptions[ idx ].setVisible( true );
                                mainFrame.setEnabled( false );
                                mainFrame.setVisible( false );
                             }
                         } );
          }
     }
  }
}
