package clarion.options;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to represent an generic option in a CLARION configuration.
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public Option( String optLabel, int choiceNum, int defaultChoice, int
 * userChoice ): the constructor.
 * <li> public Object clone(): the routine of clone this option.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to represent an generic configuration option of CLARION model.
 * <li> Usage of this class. <br>
 * It is instantiated inside any GUI object where the representation of an generic
 * option is required.
 * </ul>
 * @author Xi Zhang
 *
 */

public class Option {

  /** the option label. */
  protected String optLabel;
  /** # of items in the option. */
  protected int choiceNum;
  /** the default choice. */
  protected int defaultChoice;
  /** user choice.  */
  protected int userChoice;
  /** option items. */
  protected OptionItem optItems[];

  /** constructor.
   *  @param optLabel the option label.
   *  @param defaultChoice the default choice.
   *  @param userChoice the user's choice. */
  public Option( String optLabel, int choiceNum, int defaultChoice, int userChoice )
  {
      this.optLabel = optLabel;
      this.choiceNum = choiceNum;
      this.defaultChoice = defaultChoice;
      this.userChoice = userChoice;
  }

  /** Returns the option label.
   *  @return the option label. */
  public String getOptLabel()
  {
     return optLabel;
  }

  /** Returns the number of choices.
   *  @return the number of choices. */
  public int getChoiceNum()
  {
     return choiceNum;
  }

  /** Returns default choice.
   *  @return default choice. */
  public int getDefaultChoice()
  {
     return defaultChoice;
  }

  /** Returns user's choice.
   *  @return user's choice. */
  public int getUserChoice()
  {
     return userChoice;
  }

  /** Returns a specific option item.
   *  @param choice the specific choice.
   *  @return a specific option item. */
  public OptionItem getOptionItem( int choice )
  {
     return optItems[ choice ];
  }

  /** Sets the number of choices.
   *  @param num the number to set. */
  public void setChoiceNum( int num )
  {
     choiceNum = num;
  }

  /** Sets the default choice.
   *  @param choice the default choice to set. */
  public void setDefaultChoice( int choice )
  {
     defaultChoice = choice;
  }

  /** Sets the user choice.
   *  @param choice the user choice to set. */
  public void setUserChoice( int choice )
  {
     userChoice = choice;
  }

  /** Sets the option items.
   *  @param items the option items to set. */
  public void setOptionItems( OptionItem[] items )
  {
     try{
       optItems = new OptionItem[items.length];
       for( int i = 0; i < items.length; i++ ){
            optItems[i] = (OptionItem)(items[i].clone());
       }
     }
     catch( Exception e ){
        System.out.println(  e + ", clone OptionItem error in Option.java!" );
     }
  }

 /** methods from class <em>Object</em> */
  public String toString()
  {
     String str = "Option label : " + optLabel + "\n";
     str += "Option choice number : " + choiceNum + "\n";
     str += "Default choice : " + defaultChoice + "\n";
     str += "User choice : " + userChoice + "\n";
     for( int i = 0; i < choiceNum; i++ )
          str += optItems[i];
     return str;
  }

  /** methods from class <em>Object</em> */
  public Object clone()
  {
     Option obj = new Option( optLabel, choiceNum, defaultChoice, userChoice );
     obj.setOptionItems( optItems );
     return obj;
  }
}