package clarion.options;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to represent a generic item of an CLARION configuration option.
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public OptionItem( String label, Parameter[] params ): constructor.
 * <li> public OptionItem( String label, int num, Parameter[] params ): constructor.
 * <li> public Object clone(): the routine of clone this option item.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to represent a generic item of an CLARION configuration option.
 * <li> Usage of this class. <br>
 * It is instantiated inside any GUI object where the representation of an generic
 * option item is required.
 * </ul>
 * @author Xi Zhang
 *
 */

public class OptionItem  implements Cloneable {

  /** the label of this item. */
  protected String label;
  /** current number of parameters in this item. */
  protected int curParamNum;
  /** the parameter array. */
  protected Parameter[] params;

  /** constructor.
   *  @param label the item label.
   *  @param params the parameter array of this item. */
  public OptionItem( String label, Parameter[] params )
  {
     this.label = label;
     if( params != null ){
         try{
            curParamNum = params.length;
            this.params = new Parameter[ curParamNum ];
            for( int i = 0; i < curParamNum; i++ )
                 this.params[i] = (Parameter)(params[i].clone());
         }
         catch( Exception e ){
            System.out.println( "clone error 111 in OptionItem.java!" );
         }
     }
  }

  /** constructor.
   *  @param label the item label.
   *  @param num the number of parameters.
   *  @param params the parameter array of this item. */
  public OptionItem( String label, int num, Parameter[] params )
  {
     this.label = label;
     if( params != null ){
         try{
            curParamNum = num;
            this.params = new Parameter[ curParamNum ];
            for( int i = 0; i < curParamNum; i++ )
                 this.params[i] = (Parameter)(params[i].clone());
         }
         catch( Exception e ){
            System.out.println( "clone error 222 in OptionItem.java!" );
         }
     }
  }

  /** Returns the item label.
   *  @return the item label. */
  public String getLabel()
  {
     return label;
  }

  /** Returns the number of parameters.
   *  @return the number of parameters. */
  public int getCount()
  {
     return curParamNum;
  }

  /** Returns parameters.
   *  @return parameters. */
  public Parameter[] getParameters()
  {
     return params;
  }

  /** Returns a specific parameter.
   *  @param idx the index to the specific parameter.
   *  @return the specific parameter. */
  public Parameter getParameter( int idx )
  {
     return params[ idx ];
  }

  /** Returns if this item has parameter or not. */
  public boolean hasParameters()
  {
     //return ( params != null );
     return curParamNum > 0;
  }

  /** Sets the number of parameters.
   *  @param num the number to set. */
  public void setCount( int num )
  {
     if( curParamNum == 0 )
         return;
     if( num <= curParamNum ){
         curParamNum = num;
         return;
     }
     Parameter[] temp = new Parameter[curParamNum];
     try{
        for( int i = 0; i < curParamNum; i++ )
             temp[i] = (Parameter)(params[i].clone());
        params = new Parameter[ num ];
        for( int i = 0; i < num; i++ )
             params[i] = (Parameter)(temp[ Math.min(i,curParamNum-1) ].clone());
        curParamNum = num;
     }
     catch( Exception e ){
        System.out.println( "clone parameter error in OptionItem.java!" );
     }
  }

  /** methods from class <em>Object</em> */
  public String toString()
  {
     String str = "Item label : " + label + "\n";
     str += "Parameter number : " + curParamNum + "\n";
     for( int i = 0; i < curParamNum; i++ )
          str += params[i];
     return str;
  }

  /** methods from class <em>Object</em> */
  public Object clone()// throws CloneNotSupportedException
  {
      OptionItem item = null;
      try{
          if( params == null )
              item = new OptionItem( label, 0, null );
          else{
              Parameter[] paras = new Parameter[ params.length ];
              for( int i = 0; i < params.length; i++ )
                   paras[ i ] = (Parameter)(params[ i ].clone());
              item = new OptionItem( label, curParamNum, paras );
         }
      }
      catch( Exception e ){
             System.out.println( e + ", clone error in OptionItem.java" );
      }
      return item;
  }
}