package clarion.options.acs;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.acs.net.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring ACS subsystem by GUI.
 * It extends the base class <em>GuiTemplate</em>.  It contains the following
 * configuration entries:  ACS Components selection, IDNs and Corresponding Rule
*  Groups, Working Memory and Goal Structure.
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void returnFrom(): update the GUI components when return from other
 * screens.
 * <li> public void addActionListeners( final JFrame mainFrame ) :  event listeners
* for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the ACS subsystem by GUI
 * (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiClarion</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class GuiACS extends GuiTemplate{

  String buttonTexts[] = {
        "Components",
        "IDNs and Corresponding Rule Groups",
        "Working Memory",
        "Goal Structure"
  };
  SelectionTemplate subSelections[];
  GuiTemplate networks;

  public GuiACS( int num )
  {
     super( num, "OPTIONS OF THE ACTION-CENTERED SYSTEM");
     setButtonTexts( buttonTexts );
     subSelections = new SelectionTemplate[4];
     subSelections[0] = new SelectionAcsComp( this, 490, 220, manager );
     if( manager.getWmOption() ){
         optButtons[2].setEnabled( true );
         subSelections[1] = new SelectionAcsWM( this, 600, 400, manager );
     }
     else
         optButtons[2].setEnabled( false );
     if( manager.getGsOption() != Global.GS_NON ){
         optButtons[3].setEnabled( true );
         subSelections[2] = new SelectionAcsGS( this, 600, 380, manager );
     }
     else
         optButtons[3].setEnabled( false );
     networks = new GuiAcsNets( 4 );
     networks.init( getFrame() );
  }

  /** update the GUI components when return from other screens. */
  public void returnFrom()
  {
     if( activeButtonIdx == 0 ){
         if( manager.getWmOption() ){
             optButtons[2].setEnabled( true );
             subSelections[1] = new SelectionAcsWM( this, 600, 400, manager );
         }
         else
             optButtons[2].setEnabled( false );
         if( manager.getGsOption() != Global.GS_NON ){
             optButtons[3].setEnabled( true );
             subSelections[2] = new SelectionAcsGS( this, 600, 380, manager );
         }
         else
             optButtons[3].setEnabled( false );
         ((GuiAcsNets)networks).updateGui();
     }
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int idx = i;
          optButtons[ idx ].addActionListener(  new  ActionListener(){
                            public void actionPerformed( ActionEvent e ){
                               activeButtonIdx = idx;
                               switch( idx ){
                                 case 0:
                                      subSelections[idx].setEnabled( true );
                                      subSelections[idx].setVisible( true );
                                      break;
                                 case 1:
                                      networks.setEnabled(true);
                                      networks.setVisible(true);
                                      break;
                                 case 2: case 3: case 4:
                                      subSelections[idx-1].setEnabled( true );
                                      subSelections[idx-1].setVisible( true );
                                      break;
                              }
                              setEnabled( false );
                            }
                    } );
     }
  }
}
