package clarion.options.acs;
import javax.swing.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Goal Structure by GUI. It contains
 * the options such as a set of user-defined goals, size of goal chunk, size of Goal
 * structure, goal chunk BLA etc.
 *
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of overall ACS networks configuration and it is instantiated in
 * <tt>GuiACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsGS extends SelectionTemplate{
  // caption
  JLabel capLabel = new JLabel();
  // parameters
  JPanel panel_1 = new JPanel();
  JLabel panelLabel_1 = new JLabel();
  JLabel optLabel_1 = new JLabel();
  JRadioButton[] goalButtons = new JRadioButton[ 2 ]; {
    for( int i = 0; i < 2; i++ )
         goalButtons[i] = new JRadioButton();
  }
  JButton[] udButtons = new JButton[2]; {
    udButtons[0] = new JButton();
    udButtons[1] = new JButton();
  }
  JLabel optLabel_2 = new JLabel();
  JTextField gcSizeField = new JTextField();
  JButton paraDimButton = new JButton();
  JLabel optLabel_4 = new JLabel();
  JTextField gSizeField = new JTextField();
  JLabel optLabel_5 = new JLabel();
  JTextField thField = new JTextField();
  // goal bla
  JPanel panel_2 = new JPanel();
  // goal action by
  JPanel panel_3 = new JPanel();
  JLabel optLabel_6 = new JLabel();
  JRadioButton[] gaButtons = new JRadioButton[ 2 ]; {
    for( int i = 0; i < 2; i++ )
         gaButtons[i] = new JRadioButton();
  }
  // return button.
  JButton returnButton = new JButton();

  public SelectionAcsGS( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int GOAL_SET = 0, SIZE_GC = 1, SIZE_G = 2;
    final int TH_GS = 3, DEC_MODE = 4, BLA_OPT = 5;
    String[] defUserTexts = { "default (none)", "user defined" };

    options = manager.getOptions( Global.ACS, Global.ACS_GS, 0, 0, 0, 0 );
    setSize( width, height );
    this.getContentPane().setLayout(null);
    initLabel( capLabel, "GOAL STRUCTURE OPTIONS", new Font("Dialog", 1, 14),
               SwingConstants.CENTER, new Rectangle(140, 0, 300, 27) );
    // panel_1
    initPanel( panel_1, new Rectangle(12, 30, 260, 300) );
    initLabel( panelLabel_1, "Options and parameters for goal structure",
               new Rectangle(4, 4, 308, 17), SystemColor.activeCaption );
    panel_1.add(panelLabel_1, null);
    // all of the goals
    Rectangle[] udRects = { new Rectangle(24, 88, 200, 20), new Rectangle(24, 108, 160, 20) };
    String[] udTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.GOAL ),
                         TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.GOAL ) };
    initUserDefinedBlock( panel_1, udButtons, udRects, udTexts );
    Rectangle[] goalRects = {new Rectangle(8, 48, 100, 17), new Rectangle(8, 68, 100, 17)};
    initButtonBlock( panel_1, optLabel_1, "the set of all the possible goals",
         new Rectangle(8,28,176,17), goalButtons, defUserTexts, goalRects, options[GOAL_SET],
         1, udButtons );
    // size of goal chunk.
    initLabel( optLabel_2, "number of parameter dimensions", new Rectangle(8, 132, 190, 17) );
    panel_1.add( optLabel_2, null );
    int sizeGC = options[SIZE_GC].getOptionItem(0).getParameter(0).getIntValue();
    initField( gcSizeField, String.valueOf(sizeGC), new Rectangle(200, 132, 24, 17), 3,
               options[SIZE_GC], 0, 0 );
    panel_1.add( gcSizeField, null );
    // param dims.
    initButton( paraDimButton, "parameter dimensions", new Rectangle(30, 156, 180, 17));
    panel_1.add( paraDimButton, null );
    // size of goal structure.
    initLabel( optLabel_4, "total capacity of goal structure", new Rectangle(8, 180, 190, 17) );
    panel_1.add( optLabel_4, null );
    int sizeG = options[SIZE_G].getOptionItem(0).getParameter(0).getIntValue();
    initField( gSizeField, String.valueOf(sizeG), new Rectangle(190, 180, 24, 17), 5,
               options[SIZE_G], 0, 0  );
    panel_1.add( gSizeField, null );
    // goal threshold.
    initLabel( optLabel_5, "goal threshold", new Rectangle(8, 204, 90, 17) );
    panel_1.add(optLabel_5, null);
    double thGoal = options[TH_GS].getOptionItem(0).getParameter(0).getDoubleValue();
    initField( thField, String.valueOf(thGoal), new Rectangle(100, 204, 30, 17), 5,
               options[TH_GS], 0, 0  );
    panel_1.add(thField, null);
    // goal action determination
    String[] texts1 = { "the top level (using only Fixed Rules)", "both level" };
    Rectangle[] rects = { new Rectangle(8, 248, 240, 17), new Rectangle(8, 268, 100, 17) };
    initButtonBlock( panel_1, optLabel_6, "goal actions by", new Rectangle( 8, 228, 96, 17),
                     gaButtons, texts1, rects, options[DEC_MODE] );
    // panel_2
    // goal BLA.
    initPanel( panel_2, new Rectangle(280, 30, 290, 210) );
    initBlaBlock( panel_2, "goal base-level activation options", options[BLA_OPT] );
    // init return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(460, 300, 110, 30) );
    this.getContentPane().add( capLabel, null );
    this.getContentPane().add( panel_1, null );
    this.getContentPane().add( panel_2, null );
    this.getContentPane().add( panel_3, null );
    this.getContentPane().add( returnButton );
  }
}
