package clarion.options.acs.net;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.options.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to show the top-level screen of configuring External ACS Networks
 * components by GUI. It extends the base class <em>GuiTemplate</em>. It contains
 * the configuration entries each corresponding to one  External ACS network.
 *
 * <li> summary of the important methods. <br>
 * <ol>
 * <li> public void updateGui(): update the GUI components of this frame only
 * resulting from upper level.
 * <li> public void update(): update the GUI components of this frame only resulting
 * from lower level.
 * <li> public void addActionListeners( final JFrame mainFrame ): event listeners
 * for the buttons.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is to control the process of configuring the external ACS networks
 * components by GUI (on-line configuration).
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>GuiAcsNets</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */


public class GuiAcsExNets extends GuiTemplate{

  String buttonTexts[];
  GuiTemplate[] networks;

  public GuiAcsExNets( int netNum )
  {
     super( Global.MAX_EX_NET_NUM, "OPTIONS OF ACS EXTERNAL IDNS AND CORRESPONDING RULE GROUPS" );
     buttonTexts = new String[ OPT_NUM ];
     for( int i = 0; i < OPT_NUM; i++ )
          buttonTexts[i] = "IDN And Corresponding Rule Group - " + String.valueOf( i + 1 );
     setButtonTexts( buttonTexts );
     for( int i = netNum; i < Global.MAX_EX_NET_NUM; i++ ){
          optButtons[i].setEnabled( false );
          optButtons[i].setVisible( false );
     }
     networks = new GuiTemplate[ Global.MAX_EX_NET_NUM ];
     for( int i = 0; i < OPT_NUM; i++ ){
          networks[i] = new GuiAcsNet( Global.EX_NET, i );
          networks[i].init( getFrame() );
     }
  }

  /**  update the GUI components of this frame only resulting from upper level.  */
  public void updateGui()
  {
     int acsNetNum = manager.getAcsExNetNum();
     if( OPT_NUM >= acsNetNum ){
         for( int i = 0; i < acsNetNum; i++ ){
              optButtons[i].setEnabled( true );
              optButtons[i].setVisible( true );
         }
         for( int i = acsNetNum; i < OPT_NUM; i++ ){
              optButtons[i].setEnabled( false );
              optButtons[i].setVisible( false );
         }
         OPT_NUM = acsNetNum;
     }
     else if( OPT_NUM < acsNetNum ){
              OPT_NUM = Math.min( Global.MAX_EX_NET_NUM, acsNetNum );
              for( int i = 0; i < OPT_NUM; i++ ){
                   optButtons[i].setEnabled( true );
                   optButtons[i].setVisible( true );
              }
              for( int i = OPT_NUM; i < Global.MAX_EX_NET_NUM; i++ ){
                   optButtons[i].setEnabled( false );
                   optButtons[i].setVisible( false );
              }
          }
  }

  /**  update the GUI components of this frame only resulting from lower level.  */
  public void update()
  {
      for( int i = 0; i < OPT_NUM; i++ )
           if( optButtons[i].isEnabled() )
               ((GuiAcsNet)networks[i]).update();
  }

  /** event listeners for the buttons. */
  public void addActionListeners( final JFrame mainFrame )
  {
     for( int i = 0; i < OPT_NUM; i++ ){
          final int idx = i;
          optButtons[ idx ].addActionListener(  new  ActionListener(){
                            public void actionPerformed( ActionEvent e ){
                               activeButtonIdx = idx;
                               networks[idx].setEnabled( true );
                               networks[idx].setVisible( true );
                               setEnabled( false );
                            }
                    } );
     }
  }
}
