package clarion.options.acs.net;
import javax.swing.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionAcsRule</tt> class and is the class
 * for configuration of Fixed Rule Type component by GUI. It contains all of the
 * options those are specific to Fixed rule type such as the conditions and actions
*  of a concrete Fixed Rule.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> protected void addNewComponents(): Add the new components specific to
 * Fixed rule type.
 * <li> public void update(): update GUI components status resulting from position
 * of this rule in the rule set.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of ACS network configuration and it is instantiated in
 * <tt>GuiAcsNet</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsFR extends SelectionAcsRule{
 final int COND = 13, ACTION = 14;
 // rules
 JPanel rulePanel = new JPanel();
 JLabel ruleLabel = new JLabel();
 JLabel[] ruleLabels = new JLabel[2]; {
    ruleLabels[0] = new JLabel();
    ruleLabels[1] = new JLabel();
 }
 JButton[][] ruleUDButtons = new JButton[2][2]; {
    for( int i = 0; i < 2; i++ )
     for( int j = 0; j < 2; j++ )
         ruleUDButtons[i][j] = new JButton();
 }
 // return buttons.
 JButton nextButton = new JButton();
 JButton backButton = new JButton();

 public SelectionAcsFR( JFrame superFrame, int width, int height, int netType, int netIdx, int setIdx,
                        TransferManager manager )
 {
    super( superFrame, width, height, netType, netIdx, Global.FR, setIdx, manager );
    // add new components.
    addNewComponents();
    // update
    update();
 }

/** update GUI components status resulting from position of this rule in the rule
 *  set. */
 public void update()
 {
    super.update();
    if( setIdx > 0 )
        backButton.setEnabled( true );
    else
        backButton.setEnabled( false );
    if( setIdx < manager.getAcsCompFileNum( netType, netIdx, Global.FR )-1 )
        nextButton.setEnabled( true );
    else
        nextButton.setEnabled( false );
 }

 /** Add the new components specific to Fixed Rule type. */
 protected void addNewComponents()
 {
    // FR rules.
    initPanel( rulePanel, BORDER, null, new Rectangle(644, 344, 312, 156));
    initLabel( ruleLabel,"Fixed rule Information", new Rectangle(4, 4, 160, 17));
    String[] ruleTexts = { "rule condition", "rule action" };
    for( int i = 0; i < 2; i++ ){
         initLabel( ruleLabels[i], ruleTexts[i], new Rectangle(4, 24+66*i, 100, 17));
         rulePanel.add( ruleLabels[i] );
         Rectangle[] ruleUDRects = { new Rectangle(24, 44+66*i, 160, 17),
                                    new Rectangle(24, 64+66*i, 160, 17) };
         String[] ruleUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.FR_CON+i ),
                                 TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.FR_CON+i )};
         initUserDefinedBlock( rulePanel, ruleUDButtons[i], ruleUDRects, ruleUDTexts );
    }
    // return
    initReturnButton( backButton, "BACK", new Rectangle(626, 520, 110, 29), setIdx, 0 );
    initReturnButton( nextButton, "NEXT", new Rectangle(736, 520, 110, 29), setIdx, 1 );
    // add components.
    rulePanel.add(ruleLabel, null);
    rootContainer.add(rulePanel, null);
    rootContainer.add(nextButton, null);
    rootContainer.add(backButton, null);
 }
}
