package clarion.options.acs.net;
import javax.swing.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionAcsRule</tt> class and is the class
 * for configuration of RER Rule Type component by GUI. It contains all of the
 * options those are specific to RER rule type such as density, probability of
 * extraction, generalization/specialization mode, positivity and IG measure etc.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> protected void addNewComponents(): Add the new components specific to
*  RER rule type.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of ACS network configuration and it is instantiated in
*  <tt>GuiAcsNet</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAcsRer extends SelectionAcsRule{
 final int DEN = 13, EXT_PROB = 14, GS_MODE = 15, POS = 16;
 final int IG = 17, TH = 18, RANGE = 19, MERGE = 20;

 // parameter
 JPanel paramPanel = new JPanel();
 JLabel paramLabel = new JLabel();
 JLabel exProbLabel = new JLabel();
 JTextField exProbField = new JTextField();
 JLabel denLabel = new JLabel();
 JTextField denField = new JTextField();
 JLabel mergeLabel = new JLabel();
 JTextField mergeField = new JTextField();
 JLabel sgLabel = new JLabel();
 JRadioButton[] sgButtons = new JRadioButton[4];{
   for( int i = 0; i < 4; i++ )
       sgButtons[i] = new JRadioButton();
 }
 JLabel rangeLabel = new JLabel();
 JRadioButton[] rangeButtons = new JRadioButton[3]; {
     for( int i = 0; i < 3; i++ )
          rangeButtons[i] = new JRadioButton();
 }
 // positivity
 JPanel posPanel = new JPanel();
 JLabel posLabel = new JLabel();
 JRadioButton[] posButtons = new JRadioButton[3];{
   posButtons[0] = new JRadioButton();
   posButtons[1] = new JRadioButton();
   posButtons[2] = new JRadioButton();
 }
 JLabel posLabel11 = new JLabel();
 JLabel posLabel12 = new JLabel();
 JLabel posLabel21 = new JLabel();
 JButton[] posUDButtons = new JButton[2]; {
   posUDButtons[0] = new JButton();
   posUDButtons[1] = new JButton();
 }
 // IG
 JPanel igPanel = new JPanel();
 JLabel igLabel = new JLabel();
 JRadioButton[] igButtons = new JRadioButton[2]; {
      igButtons[0] = new JRadioButton();
      igButtons[1] = new JRadioButton();
 }
 JButton[] igUDButtons = new JButton[2]; {
    igUDButtons[0] = new JButton();
    igUDButtons[1] = new JButton();
 }
 JLabel[] igLabels = new JLabel[4]; {
    for( int i = 0; i < 4; i++ )
         igLabels[i] = new JLabel();
 }
 JTextField[][] igFields = new JTextField[1][2];{
    for( int i = 0; i < 1; i++ )
      for( int j = 0; j < 2; j++ )
          igFields[i][j] = new JTextField();
 }
 // threholds.
 JPanel rlthPanel = new JPanel();
 JLabel rlthLabel = new JLabel();
 JLabel[] rlthLabels = new JLabel[3]; {
     for( int i = 0; i < 3; i++ )
          rlthLabels[i] = new JLabel();
 }
 JTextField[] rlthFields = new JTextField[3];{
     for( int i = 0; i < 3; i++ )
          rlthFields[i] = new JTextField();
 }

 public SelectionAcsRer( JFrame superFrame, int width, int height, int netType, int netIdx,
                         int setIdx, TransferManager manager )
 {
    super( superFrame, width, height, netType, netIdx, Global.RER, setIdx, manager );
    // add new components.
    addNewComponents();
    // update
    update();
 }

 /** Add the new components specific to RER rule type. */
 protected void addNewComponents()
 {
    // parameters.
    initPanel( paramPanel, BORDER, null, new Rectangle(660, 30, 360, 210));
    initLabel( paramLabel,"RER Parameters", new Rectangle(4, 4, 123, 17));
    initLabel( denLabel,"the density parameter", new Rectangle(8, 28, 136, 17));
    initField( denField, String.valueOf(options[DEN].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(150, 28, 32, 17), 4, options[DEN], 0, 0 );
    initLabel( exProbLabel,"the rule extraction probability",new Rectangle(8, 48, 180, 17));
    initField( exProbField, String.valueOf(options[EXT_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(190, 48, 32, 17), 4, options[EXT_PROB], 0, 0 );
    String[] sgTexts ={ "none", "specific-to-general", "general-to-specific", "both" };
    Rectangle[] sgRects ={new Rectangle(16, 88, 60, 17), new Rectangle(170, 88, 150, 17),
                          new Rectangle(16, 108, 150, 17), new Rectangle(170, 108, 60, 17)};
    initButtonBlock( paramPanel, sgLabel, "generalization/specialization direction",
       new Rectangle(8, 68, 240, 17), sgButtons, sgTexts, sgRects, options[GS_MODE] );
    initLabel( mergeLabel, "merge frequency (# of steps)", new Rectangle(8, 128, 180, 17));
    initField( mergeField, String.valueOf(options[MERGE].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(190, 128, 32, 17), 4, options[MERGE], 0, 0 );
    String[] rangeTexts = {"contiguous value range","non-contiguous value range","one value or all"};
    Rectangle[] rangeRects = { new Rectangle(190, 168, 168, 17), new Rectangle(4, 168, 188, 17),
                               new Rectangle(4, 188, 140, 17) };  // default first.
    initButtonBlock( paramPanel, rangeLabel,"Generalization/specification style",
             new Rectangle(4, 148, 272, 17), rangeButtons, rangeTexts, rangeRects,options[RANGE] );
    // positivity criterion.
    initPanel( posPanel, BORDER, null, new Rectangle(660, 245, 360, 190));
    Rectangle[] posUDRects = { new Rectangle(24, 144, 160, 20), new Rectangle(24, 168, 160, 20) };
    String[] posUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.RER_POS ),
                            TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.RER_POS ) };
    initUserDefinedBlock( posPanel, posUDButtons, posUDRects, posUDTexts );
    String[] posTexts = { "default 1", "default 2", "user defined criterion"};
    Rectangle[] posRects = { new Rectangle(8, 24, 205, 17), new Rectangle(8, 84, 144, 17),
                             new Rectangle(8, 124, 164, 17) };
    initButtonBlock( posPanel, posLabel, "Rule positivity criterion", new Rectangle(4, 2, 156, 17),
                     posButtons, posTexts, posRects, options[POS], 2, posUDButtons );
    initLabel( posLabel11,"reward + QdiscountRate * maxbQ(y,b) -Q(x,a)",new Rectangle(24, 44, 259, 17));
    initLabel( posLabel12, "> threshold_RER", new Rectangle(24, 64, 110, 17));
    initLabel( posLabel21,"reward > threshold_RER", new Rectangle(24, 104, 259, 17));
    // info gain
    initPanel( igPanel, BORDER, null, new Rectangle(660, 440, 360, 175));
    Rectangle[] igUDRects = { new Rectangle(24, 128, 160, 17), new Rectangle(24, 148, 160, 17) };
    String[] igUDTexts = { TaskClarion.getSample( TaskClarion.ACS, TaskClarion.RER_IG ),
                         TaskClarion.getExplanation( TaskClarion.ACS, TaskClarion.RER_IG ) };
    initUserDefinedBlock( igPanel, igUDButtons, igUDRects, igUDTexts );
    String[] igButtonTexts = {"the default measure", "user defined measure"};
    Rectangle[] igButtonRects = {new Rectangle(8, 24, 144, 17),new Rectangle(8, 108, 164, 17)};
    initButtonBlock(igPanel,igLabel,"IG measures for rule learning ", new Rectangle(4, 2, 188, 17),
                    igButtons, igButtonTexts, igButtonRects, options[IG], igFields, 1, igUDButtons );
    String[] igLabelTexts = { "c1 =", "c2 =",
                  "IG( C, all ) = log2( ( PMa(C) + C1 ) / ( PMa(C) + NMa(C) + C2 ) )",
                  "-  log2( ( PMa(all) + C1 ) / ( PMa(all) + NMa(all) + C2 ) )"};
    Rectangle[] igLabelRects = {
                  new Rectangle(24, 44, 24, 17), new Rectangle(92, 44, 24, 17),
                  new Rectangle(16, 64, 332, 17), new Rectangle(16, 84, 300, 17)
                };
    String[] igFieldTexts = { String.valueOf(options[IG].getOptionItem(0).getParameter(0).getDoubleValue()),
                              String.valueOf(options[IG].getOptionItem(0).getParameter(1).getDoubleValue()) };
    Rectangle[] igFieldRects = {new Rectangle(50, 44, 24, 17),new Rectangle(118, 44, 24, 17)};
    initFieldBlock( igPanel, igLabels, igLabelTexts, igLabelRects, igFields[0], igFieldTexts,
                    igFieldRects, options[IG], 0 );
    // thresholds.
    initPanel( rlthPanel, BORDER, null, new Rectangle(660, 625, 360, 95));
    initLabel( rlthLabel, "Rule learning thresholds", new Rectangle(4, 2, 140, 17) );
    String[] rlthLabelTexts = {
                  "threshold_RER ( for positivity ) =", "threshold1 ( for generalization ) =",
                  "threshold2 ( for specialization ) ="
                };
    Rectangle[] rlthLabelRects = {
                  new Rectangle(8, 24, 200, 17), new Rectangle(8, 44, 200, 17),
                  new Rectangle(8, 64, 200, 17)
             };
    String[] rlthFieldTexts = {
                 String.valueOf(options[TH].getOptionItem(0).getParameter(0).getDoubleValue()),
                 String.valueOf(options[TH].getOptionItem(0).getParameter(1).getDoubleValue()),
                 String.valueOf(options[TH].getOptionItem(0).getParameter(2).getDoubleValue())
             };
    Rectangle[] rlthFieldRects = {
                new Rectangle(192, 24, 32, 17), new Rectangle(204, 44, 32, 17),
                new Rectangle(200, 64, 32, 17)
              };
    initFieldBlock( rlthPanel, rlthLabels, rlthLabelTexts, rlthLabelRects, rlthFields,
                    rlthFieldTexts, rlthFieldRects, options[TH], 0 );
    // add components.
    paramPanel.add(paramLabel, null);
    paramPanel.add(exProbLabel, null);
    paramPanel.add(exProbField, null);
    paramPanel.add(denLabel, null);
    paramPanel.add(denField, null);
    paramPanel.add(mergeLabel, null);
    paramPanel.add(mergeField, null);
    paramPanel.add(rangeLabel, null);
    posPanel.add(posLabel, null);
    posPanel.add(posLabel11, null);
    posPanel.add(posLabel12, null);
    posPanel.add(posLabel21, null);
    igPanel.add(igLabel, null);
    rlthPanel.add(rlthLabel, null);
    rootContainer.add(paramPanel, null);
    rootContainer.add(posPanel, null);
    rootContainer.add(igPanel, null);
    rootContainer.add(rlthPanel, null);
 }
}
