package clarion.options.nacs;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Abstract Episodic Memory by GUI.
 * It contains the options such as maximal of state and action that can be coded,
 * quantization of reinforcement, parameters for using EM to train AEM,
 * parameters for using AEM to train ACS and parameters for the probability networks
 * in it.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of NACS configuration and it is instantiated in
 * <tt>GuiNACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAem extends SelectionTemplate{
  JFrame superFrame;
  JLabel capLabel = new JLabel();
 // AEM
 JPanel aemPanel = new JPanel();
 JLabel aemLabel = new JLabel();
 JLabel aemStateBoundLabel = new JLabel();
 JTextField aemStateBoundField = new JTextField();
 JLabel aemActionBoundLabel = new JLabel();
 JTextField aemActionBoundField = new JTextField();
 JLabel aemQuanLabel = new JLabel();
 JLabel[] aemReinLabels = new JLabel[3]; {
    for( int i =  0; i < aemReinLabels.length; i++ )
         aemReinLabels[i] = new JLabel();
 }
 JTextField[] aemReinFields = new JTextField[3]; {
    for( int i = 0; i < aemReinFields.length; i++ )
         aemReinFields[i] = new JTextField();
 }
 JLabel aemProbLabel = new JLabel();
 JTextField aemProbField = new JTextField();
 JLabel aemItemNumLabel1 = new JLabel();
 JLabel aemItemNumLabel2 = new JLabel();
 JTextField aemItemNumField = new JTextField();
 JLabel aemIterNumLabel1 = new JLabel();
 JLabel aemIterNumLabel2 = new JLabel();
 JTextField aemIterNumField = new JTextField();
 JLabel aemRlLabel = new JLabel();
 JTextField aemRlField = new JTextField();
 JLabel aemMomLabel = new JLabel();
 JTextField aemMomField = new JTextField();
 // ACS
 JPanel acsPanel = new JPanel();
 JLabel acsLabel = new JLabel();
 JLabel acsTempLabel = new JLabel();
 JLabel acsProbLabel = new JLabel();
 JLabel acsNumLabel = new JLabel();
 JTextField acsTempField = new JTextField();
 JTextField acsProbField = new JTextField();
 JTextField acsNumField = new JTextField();
 JLabel acsIterLabel1 = new JLabel();
 JLabel acsIterLabel2 = new JLabel();
 JTextField acsIterField = new JTextField();
 JButton returnButton = new JButton();

  public SelectionAem( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int AEM_STATE_BOUND = 0, AEM_ACTION_BOUND = 1, AEM_NS_NUM = 2, AEM_QUAN = 3;
    final int AEM_EM_PROB = 4, AEM_EM_ITEM = 5, AEM_EM_ITER = 6, AEM_EM_LR = 7;
    final int AEM_EM_MOM = 8, AEM_ACS_PROB = 9, AEM_ACS_ITEM = 10;
    final int AEM_ACS_ITER = 11, AEM_ACS_TEMP = 12;

    options = manager.getOptions( Global.NACS, Global.NACS_AEM, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout(null);
    // caption
    initLabel( capLabel,"AEM   OPTIONS",new java.awt.Font("Dialog", 1, 16),
               SwingConstants.CENTER, new Rectangle(50, 0, 240, 26));
    // training AEM
    initPanel( aemPanel, BORDER, null, new Rectangle(10, 26, 380, 260));
    initLabel( aemLabel, "Parameters for training AEM", new Rectangle(4, 6, 320, 17) );

    initLabel( aemStateBoundLabel, "Max # of states that can be coded", new Rectangle(8, 28, 200, 17));
    initField( aemStateBoundField, String.valueOf(options[AEM_STATE_BOUND].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle( 210, 28, 32, 17), 4, options[AEM_STATE_BOUND], 0, 0 );

    initLabel( aemActionBoundLabel, "Max # of actions that can be coded", new Rectangle(8, 48, 200, 17));
    initField( aemActionBoundField, String.valueOf(options[AEM_ACTION_BOUND].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle( 210, 48, 32, 17), 4, options[AEM_ACTION_BOUND], 0, 0 );

    initLabel( aemQuanLabel, "The range and increment for quantization of renforcement", new Rectangle(8, 68, 340, 17));
    String[] aemReinLabelText = { "lower bound", "upper bound", "increment"};
    Rectangle[] aemReinLabelRects = { new Rectangle(8, 88, 80, 17), new Rectangle(130, 88, 80, 17),
                                      new Rectangle(250, 88, 80, 17) };
    String[] aemReinFieldTexts = {
                 String.valueOf(options[AEM_QUAN].getOptionItem(0).getParameter(0).getDoubleValue()),
                 String.valueOf(options[AEM_QUAN].getOptionItem(0).getParameter(1).getDoubleValue()),
                 String.valueOf(options[AEM_QUAN].getOptionItem(0).getParameter(2).getDoubleValue())
             };
    Rectangle[] aemReinFieldRects = { new Rectangle(85, 88, 32, 17), new Rectangle(205, 88, 32, 17),
                                      new Rectangle(310, 88, 32, 17) };
    initFieldBlock( aemPanel, aemReinLabels, aemReinLabelText, aemReinLabelRects,
                    aemReinFields, aemReinFieldTexts, aemReinFieldRects, options[AEM_QUAN], 0 );

    initLabel( aemProbLabel, "probability of training AEM at each step", new Rectangle(8, 108, 221, 17));
    initField( aemProbField, String.valueOf(options[AEM_EM_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(235, 108, 32, 17), 4, options[AEM_EM_PROB], 0, 0 );

    initLabel( aemItemNumLabel1, "number of action-centered EM items to pick for training AEM",
               new Rectangle(8, 128, 350, 17));
    initLabel( aemItemNumLabel2, "at each step ", new Rectangle(8, 148, 88, 17));
    initField( aemItemNumField, String.valueOf(options[AEM_EM_ITEM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(90, 148, 32, 17), 4, options[AEM_EM_ITEM], 0, 0 );

    initLabel( aemIterNumLabel1, "number of times each selected item is presented to the AEM",
               new Rectangle(8, 168, 350, 17));
    initLabel( aemIterNumLabel2, "at each step", new Rectangle(8, 188, 80, 17));
    initField( aemIterNumField, String.valueOf(options[AEM_EM_ITER].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(90, 188, 32, 17), 4, options[AEM_EM_ITER], 0, 0 );

    initLabel( aemRlLabel, "BP learning rate for training AEM", new Rectangle(8, 208, 200, 17));
    initField( aemRlField, String.valueOf(options[AEM_EM_LR].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(208, 208, 32, 17), 4, options[AEM_EM_LR], 0, 0 );
    initLabel( aemMomLabel, "BP momentum for traIning AEM", new Rectangle(8, 228, 190, 17));
    initField( aemMomField, String.valueOf(options[AEM_EM_MOM].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(200, 228, 32, 17), 4, options[AEM_EM_MOM], 0, 0 );
    // training ACS
    initPanel( acsPanel, BORDER, null, new Rectangle(10, 300, 380, 150));
    initLabel( acsLabel,"Parameters for using AEM to train ACS",new Rectangle(4, 6, 221, 17));
    initLabel( acsProbLabel,"probability of using AEM to train ACS at each step", new Rectangle(8, 32, 290, 17));
    initField( acsProbField, String.valueOf(options[AEM_ACS_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(300, 32, 32, 17), 4, options[AEM_ACS_PROB], 0, 0 );
    initLabel( acsNumLabel,"number of  AEM  samples to pick for training ACS each step",
               new Rectangle(8, 54, 340, 17));
    initField( acsNumField, String.valueOf(options[AEM_ACS_ITEM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(350, 54, 24, 17), 4, options[AEM_ACS_ITEM], 0, 0 );
    initLabel( acsIterLabel1,"number of  times each selected item is presented to ACS for",
               new Rectangle(8, 76, 350, 17));
    initLabel( acsIterLabel2, "training ACS at each step", new Rectangle(8, 98, 150, 17));
    initField( acsIterField, String.valueOf(options[AEM_ACS_ITER].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(160, 98, 32, 17), 4, options[AEM_ACS_ITER], 0, 0 );
    initLabel( acsTempLabel, "temperature for output selection", new Rectangle(8, 120, 185, 17));
    initField( acsTempField, String.valueOf(options[AEM_ACS_TEMP].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(200, 120, 32, 17), 4, options[AEM_ACS_TEMP], 0, 0  );
    // RETTURN BUTTONS.
    initReturnButton( returnButton, "RETURN", new Rectangle(250, 470, 80, 29));
    aemPanel.add(aemLabel, null);
    aemPanel.add(aemStateBoundLabel, null);
    aemPanel.add(aemStateBoundField, null);
    aemPanel.add(aemActionBoundLabel, null);
    aemPanel.add(aemActionBoundField, null);
    aemPanel.add(aemQuanLabel);
    aemPanel.add(aemProbLabel, null);
    aemPanel.add(aemProbField, null);
    aemPanel.add(aemItemNumLabel1, null);
    aemPanel.add(aemItemNumLabel2, null);
    aemPanel.add(aemItemNumField, null);
    aemPanel.add(aemIterNumLabel1, null);
    aemPanel.add(aemIterNumLabel2, null);
    aemPanel.add(aemIterNumField, null);
    aemPanel.add(aemRlLabel, null);
    aemPanel.add(aemRlField, null);
    aemPanel.add(aemMomLabel, null);
    aemPanel.add(aemMomField, null);
    acsPanel.add(acsLabel, null);
    acsPanel.add(acsProbField, null);
    acsPanel.add(acsNumField, null);
    acsPanel.add(acsTempLabel, null);
    acsPanel.add(acsTempField, null);
    acsPanel.add(acsProbLabel, null);
    acsPanel.add(acsNumLabel, null);
    acsPanel.add(acsIterLabel1, null);
    acsPanel.add(acsIterLabel2, null);
    acsPanel.add(acsIterField, null);
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(aemPanel, null);
    this.getContentPane().add(acsPanel, null);
    this.getContentPane().add(returnButton, null);
    update();
  }

  public void update()
  {
      /*
      if( manager.isAemOn() ){
          aemPanel.setEnabled( true );
          for( int i = 0; i < aemPanel.getComponentCount(); i++ )
               aemPanel.getComponent(i).setEnabled( true );
          acsPanel.setEnabled( true );
          for( int i = 0; i < acsPanel.getComponentCount(); i++ )
               acsPanel.getComponent(i).setEnabled( true );
      }
      else{
          aemPanel.setEnabled( false );
          for( int i = 0; i < aemPanel.getComponentCount(); i++ )
               aemPanel.getComponent(i).setEnabled( false );
          acsPanel.setEnabled( false );
          for( int i = 0; i < acsPanel.getComponentCount(); i++ )
               acsPanel.getComponent(i).setEnabled( false );
      }
      */
  }
}
