package clarion.options.nacs;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to the overall structure of NACS subsystem
 * by GUI. It contains the options such as selections of GKS, number of AMN
 * networks, selections of Episodic Memory (EM) and Abstract EM.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of overall NACS networks configuration and it is instantiated in
 * <tt>GuiNACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionNacsComp extends SelectionTemplate{
  JFrame superFrame;
  JLabel capLabel = new JLabel();
  JPanel compPanel = new JPanel();
  JLabel comLabel = new JLabel();
  JLabel gksLabel = new JLabel();
  JRadioButton[] gksButtons = new JRadioButton[2]; {
     gksButtons[0] = new JRadioButton();
     gksButtons[1] = new JRadioButton();
  }
  JLabel amnLabel = new JLabel();
  JTextField amnField = new JTextField();
  JLabel emLabel = new JLabel();
  JRadioButton[] emButtons = new JRadioButton[2]; {
     emButtons[0] = new JRadioButton();
     emButtons[1] = new JRadioButton();
  }
  JLabel aemLabel = new JLabel();
  JRadioButton[] aemButtons = new JRadioButton[2]; {
     aemButtons[0] = new JRadioButton();
     aemButtons[1] = new JRadioButton();
  }
  JButton returnButton = new JButton();

  public SelectionNacsComp( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame.  */
  public void initGui()
  {
    final int GKS = 0, AMN = 1, EM = 2, AEM = 3;

    options = manager.getOptions( Global.NACS, Global.NACS_COMP, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout(null);
    // caption
    initLabel( capLabel,"OPTIONS OF NACS COMPONENTS",new java.awt.Font("Dialog", 1, 17),
               SwingConstants.CENTER, new Rectangle(19, 2, 344, 27));
    // component panel
    initPanel( compPanel, BORDER, null, new Rectangle(10, 32, 367, 159));
    initLabel( comLabel, "NACS components", new Rectangle(11, 8, 116, 17) );
    // GKS
    Rectangle[] gksRects = {new Rectangle(59, 34, 50, 17),new Rectangle(111, 34, 55, 17)};
    initButtonBlock( compPanel, gksLabel, "GKS",new Rectangle(12, 34, 40, 17),
                     gksButtons, onOffTexts, gksRects, options[GKS] );
    // AMN
    initLabel( amnLabel, "number of AMN networks",new Rectangle(10, 56, 160, 17));
    initField( amnField, String.valueOf(options[AMN].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(170, 56, 20, 17), 5, options[AMN], 0, 0 );
    // EM
    Rectangle[] emRects = {new Rectangle(128, 81, 46, 17),new Rectangle(180, 81, 51, 17)};
    initButtonBlock( compPanel, emLabel, "Episodic Memory", new Rectangle(12, 81, 110, 17),
                     emButtons, onOffTexts, emRects, options[EM] );
    // AEM
    Rectangle[] aemRects = {new Rectangle(166, 110, 46, 17),new Rectangle(217, 111, 51, 17)};
    initButtonBlock( compPanel, aemLabel, "Abstract Episodic Memory", new Rectangle(11, 109, 156, 17),
                     aemButtons, onOffTexts, aemRects, options[AEM] );
    // RETTURN BUTTONS.
    initReturnButton( returnButton, "RETURN", new Rectangle(292, 208, 80, 28));
    compPanel.add(comLabel, null);
    compPanel.add(amnLabel, null);
    compPanel.add(amnField, null);
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(compPanel, null);
    this.getContentPane().add(returnButton, null);
  }
}
