package clarion.options.nacs.amn;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to one AMN network in the bottom level
 * of NACS subsystem, by GUI. It contains the options such as I/O dimension information,
 * number of passes in one round of reasoning, parameters for network learning,
 * parameters for using EM to train AMN.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of NACS configuration and it is instantiated in
 * <tt>GuiNACS</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionAmn extends SelectionTemplate{
 int netIdx;

 JFrame superFrame;
 // caption
 JLabel capLabel = new JLabel();
 // general
 JPanel genPanel = new JPanel();
 JLabel ioLabel = new JLabel();
 JLabel amnTypeLabel = new JLabel();
 JRadioButton[] amnTypeButtons = new JRadioButton[2]; {
    amnTypeButtons[0] = new JRadioButton();
    amnTypeButtons[1] = new JRadioButton();
 }
 JButton inDimButton = new JButton();
 JButton outDimButton = new JButton();
 JLabel hideUnitLabel = new JLabel();
 JTextField hideUnitField = new JTextField();
 // pass
 JPanel passPanel = new JPanel();
 JLabel passLabel = new JLabel();
 JRadioButton[] passButtons = new JRadioButton[2]; {
    passButtons[0] = new JRadioButton();
    passButtons[1] = new JRadioButton();
 }
 JLabel passNumLabel = new JLabel();
 JTextField passNumField = new JTextField();
 // AAM or HAM
 JPanel assimPanel = new JPanel();
 JLabel assimLabel = new JLabel();
 JLabel assimProbLabel = new JLabel();
 JTextField assimProbField = new JTextField();
 JLabel assimNumLabel = new JLabel();
 JTextField assimNumField = new JTextField();
 JLabel assimIterLabel = new JLabel();
 JTextField assimIterField = new JTextField();
 JLabel assimTypeLabel = new JLabel();
 JRadioButton[] assimTypeButtons = new JRadioButton[7]; {
    for(int i = 0; i < 7; i++ )
        assimTypeButtons[i] = new JRadioButton();
 }
 // learn
 JPanel learnPanel = new JPanel();
 JLabel learnLabel = new JLabel();
 JLabel lrLabel = new JLabel();
 JTextField lrField = new JTextField();
 JLabel momLabel = new JLabel();
 JTextField momField = new JTextField();
 // return.
 JButton returnButton = new JButton();
 OptionDims[] optDims = new OptionDims[2];

 public SelectionAmn( JFrame superFrame, int width, int height, int netIdx, TransferManager manager )
 {
    super( superFrame, width, height, manager );
    this.netIdx = netIdx;
    initGui();
 }

 /** initialize the GUI components in this frame. */
 public void initGui()
 {
    final int IN_DIM = 0, OUT_DIM = 1;
    final int UNITS = 2, PASS = 3, L_RATE = 4, MOM = 5;
    final int TRAIN_PROB = 6, PATTERN_NUM = 7, ITER_NUM = 8, AAM_TYPE = 9;

    options = manager.getOptions( Global.NACS, Global.NACS_AMN, netIdx, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    String[] onOffTexts = { "ON", "OFF" };
    setSize( width, height );
    this.getContentPane().setLayout(null);
    // caption
    initLabel( capLabel, "OPTIONS FOR EACH AMN ( AMN # = " + netIdx + " )", new java.awt.Font("Dialog", 1, 14),
               SwingConstants.CENTER, new Rectangle((width-300)/2, 0, 300, 26) );
    // general
    initPanel( genPanel, BORDER, null, new Rectangle(10, 26, 440, 100));
    initLabel( ioLabel, "AMN general options", new Rectangle(4, 4, 120, 17));
    initButton( inDimButton, "dimensions in input", new Rectangle( 20, 38, 150, 20));
    initButton( outDimButton, "dimensions in output", new Rectangle( 190, 38, 160, 20));
    // frames for dimension selection
    optDims[0] = new OptionDims( this, 420, 780, "IDN INPUT SPECIFICATION", 30, manager,
        options[IN_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0], options[IN_DIM] );
    optDims[1] = new OptionDims( this, 420, 780, "IDN OUTPUT SPECIFICATION", 30, manager,
        options[OUT_DIM].getOptionItem(0).getParameter(Global.DIM_SPEC_TYPE).getIntValues()[0], options[OUT_DIM] );
    initDimButton( this, inDimButton, optDims[0] );
    initDimButton( this, outDimButton, optDims[1] );
    initLabel( hideUnitLabel, "Number of hidden units", new Rectangle(8, 68, 328, 17));
    initField( hideUnitField, String.valueOf( options[UNITS].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(144, 68, 32, 17), 4, options[UNITS], 0, 1 );
    // pass
    initPanel( passPanel, BORDER, null, new Rectangle(10, 130, 440, 50));
    String[] passTexts = {"one pass", "multiple pass",};
    Rectangle[] passRects = {new Rectangle(160, 4, 80, 17), new Rectangle(240, 4, 140, 17)};
    initButtonBlock( passPanel, passLabel, "Options for using AMN", new Rectangle(4, 4, 144, 17),
                     passButtons, passTexts, passRects, options[PASS], 1, passNumField );
    initLabel( passNumLabel,"if multiple pass, the number of iterations", new Rectangle(8, 24, 240, 17));
    initField( passNumField, String.valueOf( options[PASS].getOptionItem(1).getParameter(0).getIntValue()),
               new Rectangle(250, 24, 32, 17));
    // BP learning in AMN
    initPanel( learnPanel, BORDER, null, new Rectangle(10, 184, 440, 46));
    initLabel( learnLabel,"Parameters for BP learning in AMN", new Rectangle(4, 4, 202, 17) );
    initLabel( lrLabel, "BP learning rate", new Rectangle(8, 24, 96, 17));
    initField( lrField, String.valueOf( options[L_RATE].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(104, 24, 32, 17), 4, options[L_RATE], 0, 0 );
    initLabel( momLabel, "BP momentum", new Rectangle(176, 24, 96, 17));
    initField( momField, String.valueOf( options[MOM].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(272, 24, 32, 17), 4, options[MOM], 0, 0 );
    // AAM or HAM
    initPanel( assimPanel, BORDER, null, new Rectangle(10, 234, 440, 250));
    initLabel( assimLabel, "Options and parameters for AMN assimilation", new Rectangle(4, 4, 258, 17) );
    initLabel( assimProbLabel,"probability of the network training at each step",new Rectangle(8, 26, 270, 17));
    initField( assimProbField, String.valueOf(options[TRAIN_PROB].getOptionItem(0).getParameter(0).getDoubleValue()),
               new Rectangle(280, 28, 32, 17), 4, options[TRAIN_PROB], 0, 0 );
    initLabel( assimNumLabel, "number of items  from EM to use for training this AMN at each step",
               new Rectangle(8, 48, 380, 17));
    initField( assimNumField, String.valueOf(options[PATTERN_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(390, 48, 32, 17), 4, options[PATTERN_NUM], 0, 0 );
    initLabel( assimIterLabel, "number of training iterations for the chosen training set at each step",
               new Rectangle(8, 68, 390, 17) );
    initField( assimIterField, String.valueOf(options[ITER_NUM].getOptionItem(0).getParameter(0).getIntValue()),
               new Rectangle(400, 68, 32, 17), 4, options[ITER_NUM], 0, 0 );
    String[] assimTypeTexts = {
               "all associative rules applied",
               "all associations given by the ACS",
               "all associations given by the ACS for this particular AMN",
               "all associations inferred by the GKS",
               "auto-associations of all states experienced by the ACS",
               "auto-associations of all states transitions experienced by the ACS",
               "auot-associations of the associations given by the ACS" };
    Rectangle[] assimTypeRects = {
                   new Rectangle(12, 108, 400, 17),
                   new Rectangle(12, 128, 400, 17),
                   new Rectangle(12, 148, 400, 17),
                   new Rectangle(12, 168, 400, 17),
                   new Rectangle(12, 188, 400, 17),
                   new Rectangle(12, 208, 400, 17),
                   new Rectangle(12, 228, 400, 17)
                    };
    initButtonBlock( assimPanel, assimTypeLabel, "the type of items from EM to use for training this AMN at each step",
        new Rectangle(8, 88, 430, 17), assimTypeButtons, assimTypeTexts, assimTypeRects, options[AAM_TYPE]);
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(330, 500, 110, 29));
    genPanel.add(ioLabel, null);
    genPanel.add(inDimButton, null);
    genPanel.add(outDimButton, null);
    genPanel.add(hideUnitLabel, null);
    genPanel.add(hideUnitField, null);
    passPanel.add(passLabel, null);
    passPanel.add(passNumLabel, null);
    passPanel.add(passNumField, null);
    assimPanel.add(assimLabel, null);
    assimPanel.add(assimProbLabel, null);
    assimPanel.add(assimProbField, null);
    assimPanel.add(assimNumLabel, null);
    assimPanel.add(assimNumField, null);
    assimPanel.add(assimIterLabel, null);
    assimPanel.add(assimIterField, null);
    learnPanel.add(learnLabel, null);
    learnPanel.add(momLabel, null);
    learnPanel.add(lrLabel, null);
    learnPanel.add(lrField, null);
    learnPanel.add(momField, null);
    this.getContentPane().add(genPanel, null);
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(passPanel, null);
    this.getContentPane().add(assimPanel, null);
    this.getContentPane().add(learnPanel, null);
    this.getContentPane().add(returnButton, null);
 }

 public void update()
 {
 }
}
