package clarion.options.supervise;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class extends the <tt>SelectionTemplate</tt> class and is the class
 * for configuration of options relevant to Motivational Subsystem in Supervisory
 * subsystem by GUI. It contains the options such as Drives, Drive Representation,
 * parameters for Network for calculation of Drive strengths.
 *
 * <li> summary of the important methods <br>
 * <ol>
 * <li> public void initGui(): initialize the GUI components in this frame.
 * </ol>
 * </ul>
 * <li> Usage of this class. <br>
 * It is part of GUI of Supervisory subsystem configuration and it is instantiated
 * in <tt>GuiSupervisory</tt> class.
 * </ul>
 * @author Xi Zhang
 *
 */

public class SelectionMS extends SelectionTemplate{

  JLabel capLabel = new JLabel();
  // drive
  JPanel driveSetPanel = new JPanel();
  JLabel driveSetLabel = new JLabel();
  JRadioButton[] driveSetButtons = new JRadioButton[2]; {
      driveSetButtons[0] = new JRadioButton();
      driveSetButtons[1] = new JRadioButton();
  }
  JButton[] driveSetUDButtons = new JButton[2]; {
    driveSetUDButtons[0] = new JButton();
    driveSetUDButtons[1] = new JButton();
  }
  // drive representation
  JPanel driveRepPanel = new JPanel();
  JLabel driveRepLabel = new JLabel();
  JRadioButton[] driveRepButtons = new JRadioButton[2]; {
      driveRepButtons[0] = new JRadioButton();
      driveRepButtons[1] = new JRadioButton();
  }
  // drive representation network
  boolean[] driveNetOn;
  JPanel driveNetPanel = new JPanel();
  JLabel driveNetLabel = new JLabel();
  JButton inputDimButton = new JButton();
  JButton outputDimButton = new JButton();
  JLabel hideUnitLabel = new JLabel();
  JTextField hideUnitField = new JTextField();
  JLabel blTempLabel = new JLabel();
  JTextField blTempField = new JTextField();
  JLabel lrLabel = new JLabel();
  JTextField lrField = new JTextField();
  JLabel momLabel = new JLabel();
  JTextField momField = new JTextField();
  // return button
  JButton returnButton = new JButton();

  public SelectionMS( JFrame superFrame, int width, int height, TransferManager manager )
  {
    super( superFrame, width, height, manager );
    initGui();
  }

  /** initialize the GUI components in this frame. */
  public void initGui()
  {
    final int DRIVE = 0, DRIVE_REP = 1, DRIVE_NET = 2;

    options = manager.getOptions( Global.SUPER, Global.SUPER_MS, 0, 0, 0, 0 );
    Border BORDER = BorderFactory.createLineBorder(Color.black);
    Color COLOR = SystemColor.activeCaption;
    setSize( width, height );
    this.getContentPane().setLayout( null );
    // caption.
    initLabel( capLabel, "OPTIONS FOR MOTIVATIONAL SUBSYSTEM", new Font("Dialog",1,14),
               SwingConstants.CENTER, new Rectangle( 100, 8, 500, 30) );
    // drive set.
    initPanel( driveSetPanel, BORDER, null, new Rectangle(10, 40, 360, 120) );
    Rectangle[] driveSetUDRects = { new Rectangle(24, 68, 160, 20), new Rectangle(24, 90, 160, 20) };
    String[] driveSetUDTexts = { TaskClarion.getSample( TaskClarion.MCS, TaskClarion.DRIVE ),
                               TaskClarion.getExplanation( TaskClarion.MCS, TaskClarion.DRIVE ) };
    initUserDefinedBlock( driveSetPanel, driveSetUDButtons, driveSetUDRects, driveSetUDTexts );
    String[] driveSetTexts = {"default drive set", "user defined"};
    Rectangle[] driveSetRects = {new Rectangle(4, 28, 240, 17),new Rectangle(4, 48, 100, 17)};
    initButtonBlock( driveSetPanel, driveSetLabel, "Drives", new Rectangle(4, 4, 230, 17), driveSetButtons,
                     driveSetTexts, driveSetRects, options[DRIVE], 1, driveSetUDButtons );
    // Parameters for the drive computation BP network.
    initPanel( driveNetPanel, BORDER, null, new Rectangle(380, 40, 350, 160));
    initLabel( driveNetLabel,"Parameters for BP learning",new Rectangle(4, 4, 210, 17) );
    initButton( inputDimButton, "dimensions in input", new Rectangle(4, 30, 160, 24));
    initButton( outputDimButton, "dimensions in output", new Rectangle(170, 30, 170, 24));
    initLabel( hideUnitLabel, "Number of hidden units", new Rectangle(4, 60, 134, 17));
    initField( hideUnitField, String.valueOf( options[DRIVE_NET].getOptionItem(0).getParameter(1).getIntValue()),
               new Rectangle(144, 60, 32, 17), 4, options[DRIVE_NET], 0, 1 );
    initLabel( lrLabel, "BP learning rate", new Rectangle(4, 80, 96, 17) );
    initField( lrField, String.valueOf( options[DRIVE_NET].getOptionItem(0).getParameter(2).getDoubleValue()),
               new Rectangle(100, 80, 32, 17), 4, options[DRIVE_NET], 0, 2 );
    initLabel( momLabel, "BP momentum", new Rectangle(4, 100, 96, 17));
    initField( momField, String.valueOf( options[DRIVE_NET].getOptionItem(0).getParameter(3).getDoubleValue()),
               new Rectangle(100, 100, 32, 17), 4, options[DRIVE_NET], 0, 3 );
    initLabel( blTempLabel, "Boltzmann temperature for bottom-level action", new Rectangle(4, 120, 280, 17));
    initField( blTempField, String.valueOf( options[DRIVE_NET].getOptionItem(0).getParameter(4).getDoubleValue()),
               new Rectangle(290, 120, 32, 17), 4, options[DRIVE_NET], 0, 4 );
    driveNetPanel.add( driveNetLabel );
    driveNetPanel.add( inputDimButton );
    driveNetPanel.add( outputDimButton );
    driveNetPanel.add( hideUnitLabel );
    driveNetPanel.add( hideUnitField );
    driveNetPanel.add( lrLabel );
    driveNetPanel.add( lrField );
    driveNetPanel.add( momLabel );
    driveNetPanel.add( momField );
    driveNetPanel.add( blTempLabel );
    driveNetPanel.add( blTempField );
    driveNetOn = new boolean[driveNetPanel.getComponentCount()];
    this.initPanelStatus( options[DRIVE_REP], 1, driveNetPanel, driveNetOn );
    // drive representation
    initPanel( driveRepPanel, BORDER, null, new Rectangle(10, 170, 360, 90) );
    String[] driveRepTexts = { "drive computation using given set of equations",
                               "drive computation using a pre-trained neural network" };
    Rectangle[] driveRepRects = {new Rectangle(4, 28, 300, 17),new Rectangle(4, 58, 320, 17)};
    initButtonBlock( driveRepPanel, driveRepLabel, "Drive Representation", new Rectangle(4, 4, 240, 17),
          driveRepButtons, driveRepTexts, driveRepRects, options[DRIVE_REP], 1, driveNetPanel, driveNetOn);
    // return button.
    initReturnButton( returnButton, "RETURN", new Rectangle(600, 220, 100, 28) );
    this.getContentPane().add(capLabel, null);
    this.getContentPane().add(driveSetPanel, null);
    this.getContentPane().add(driveRepPanel, null);
    this.getContentPane().add(driveNetPanel, null );
    this.getContentPane().add(returnButton, null);
 }
}
