package clarion.options.transfer.subsys;
import java.io.*;
import clarion.system.*;
import clarion.options.*;
import clarion.options.transfer.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to implement the process of transferring Task-Specific configuration
 * data between GUI and <tt>Global</tt> object or persistent storage.
 * <li> important methods <br>
 * <ol>
 * <li> public void load(): implement the process of loading options from persistent
 * storage.
 * <li> public void save(): implement the process of save the options to persistent
 * storage or to the associated <tt>Global</tt> object.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class is part of data transfer subsystem and is not related directly to
 * CLARION model.
 * <li> Usage of this class. <br>
 * It extends the base <tt>Transfer</t>> class and is instantiated in the class
 * <em>TransferManager</em>.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class TaskSpecTransfer extends Transfer {

  /** constructor. instantiated with the associated <tt>TransferManager</tt> object
   * and <tt>Global</tt> object. */
  public TaskSpecTransfer( TransferManager manager, Global global )
  {
     super( manager, global );
  }

  /** implement the process of loading options from persistent storage. */
  public void load()
  {
     loadedFileName = "taskspec.txt";
     String path = global.getSettingDir() + "\\DEFAULT\\subjectGroup0\\taskspec\\";
     sysOptions = new Option[1][1][1][1][1][];
     sysOptions[0][0][0][0][0] = loadOptions( path );
     //path = global.getSettingDir() + "\\" + global.getTaskName()
     //             + "\\" + global.getTaskSettingName() + "\\taskspec\\";
     subsysPath = this.searchFile( "taskspec" );
     userOptions = new Option[1][1][1][1][1][];
     userOptions[0][0][0][0][0] = loadOptions( subsysPath );
  }

  /** implement the process of save the options to persistent storage or to the
   * associated <tt>Global</tt> object. */
  public void save()
  {
     if( saveCode == SAVE_TO_GLOBAL ){
         initGlobalTaskSpec();
         return;
     }
     // create RT setting directory.
     String path = global.getSettingDir() + "\\" + global.getTaskName()
                   + "\\" + global.getTaskSettingName() + "\\taskspec";
     new File( path ).mkdir();
     savedFileName = "\\taskspec.txt";
     try{
        new File( path + savedFileName ).createNewFile();
     }
     catch( Exception e ){
        System.out.println( e );
     }
     // save RT setting.
     saveOptions( path, userOptions[0][0][0][0][0] );
  }

  /** initialize the variables relevant to task-specific in the associated
   * <tt>Global</tt> object. */
 public void initGlobalTaskSpec()
  {
     final int RECORD = 0;
     Option[] opts = userOptions[0][0][0][0][0];
     boolean[] outputFormat = new boolean[ opts[RECORD].getOptionItem(0).getCount()];
     for( int i = 0; i < outputFormat.length; i++ )
          outputFormat[i] = opts[RECORD].getOptionItem(0).getParameter(0).getBoolValue();
     global.setOutputFormat( outputFormat );
  }

}
