package tasksimulators;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import clarion.system.*;
import clarion.common.*;
import clarion.options.*;
import clarion.tools.*;
import main.ClarionMain;


/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is the main interface for user to configure and execute a task.
 * In this interface, it guides user in serven steps in a fixed order. The seven
 *  steps are  1, set number of running agents, 2, choose control mode, 3. choos
 *  configuration mode (with or without GUI),  4. on-line configuration of task-
 * specific options, 5. on-line confiuguration of CLARION options. 6. choose output
 * file to save simulation data, 7. Run the configured task. After, you can repeat
 * the whole process.
 *
 * <li> summary of the important methods. <br>
 * 1. private void jbInit() throws Exception: initialize all of the graphical
*  components.
*  2. Event listeners for the graphical components.
 * </ul>
 * <li> Role in the CLARION model <br>
*  This belongs to CLARION GUI subsystem. It is the main entry into on-line
*  configuration (GUI mode) and manage the overal task simulaiton process.
 * <li> Usage of <tt>this</tt> class. <br>
*  One instance of this class will be automatically created and called by CLARION
*  system when the system is started.
 * </ul>
 * @author Xi Zhang
 */

public class NewSimulator extends JFrame {

  /* the screen of system is initially started. */
  protected ClarionMain mainFrame;
  protected boolean enterFlag;
  /* the name of the task to simulate. */
  protected String taskName;
  protected WriteFile wDataFile = new WriteFile();
  //protected RandomAccessFile dataFile;
 /** the maximal number of agents running simultaneously. */
  protected final int MAX_RUNNING_AGENT_NUM = Global.MAX_RUNNING_AGENT_NUM;
  /** the three control modes. */
  protected final int BY_HUMAN = 0, BY_CLARION = 1, BY_ACTR = 2;
  /** configuration modes: with GUI or without GUI. */
  protected final int W_GUI = 0, O_GUI = 1;
  /** the number of agents running simultaneously. */
  protected int runningAgentNum = 1;
  /** control mode: by human, by clarion, by ACT-R. */
  protected int controlMode = BY_CLARION;
  /** configuration mode: with GUI or without GUI. */
  protected int configMode = W_GUI;
  /** the GUI for CLARION configuration. */
  protected GuiClarion guiClarion;
  /** the GUI for Task-Specific configuration. */
  protected GuiTask guiTask;
  /** the class of task-specific CLARION settings. */
  protected Class taskClarionClass = Global.getTaskClarionClass();
  /** the array of <em>TaskClarion</em> objects assoiciated with the running agents. */
  protected TaskClarion[] taskClarions = new TaskClarion[MAX_RUNNING_AGENT_NUM];{
     try{
        for( int i = 0; i < runningAgentNum; i++ )
             taskClarions[i] = (TaskClarion)(taskClarionClass.newInstance());
     }
     catch( Exception e1 ){
            System.out.println( e1 );
     }
  }
  /** the array of <em>Global</em> objects assoiciated with the running agents. */
  protected Global[] globals = new Global[MAX_RUNNING_AGENT_NUM];{
     for( int i = 0; i < runningAgentNum; i++ ){
          globals[i] = new Global();
          taskClarions[i].setGlobal( globals[i] );
          globals[i].setTaskClarion( taskClarions[i] );
     }
  }
  JFrame thisFrame;

  JPanel titilePanel = new JPanel();
  JLabel taskLabel = new JLabel();
  GridLayout gridLayout1 = new GridLayout();
  JLabel copyRightLabel = new JLabel();
  JTextArea instructTextArea = new JTextArea();
  JPanel stepPanel = new JPanel();
  JLabel step1Label = new JLabel();
  JLabel step2Label = new JLabel();
  JLabel step3Label = new JLabel();
  JLabel step4Label = new JLabel();
  JLabel step5Label = new JLabel();
  JLabel step6Label = new JLabel();
  JLabel step7Label = new JLabel();
  ButtonGroup controlButtonGroup = new ButtonGroup();
  JRadioButton controlRadioButton1 = new JRadioButton();
  JRadioButton controlRadioButton2 = new JRadioButton();
  JRadioButton controlRadioButton3 = new JRadioButton();
  ButtonGroup configButtonGroup = new ButtonGroup();
  JRadioButton configRadioButton1 = new JRadioButton();
  JRadioButton configRadioButton2 = new JRadioButton();
  JTextField agentNumberTextField = new JTextField();
  JTextField outputFileTextField = new JTextField();
  JButton gotoTaskSpecificGuiButton = new JButton();
  JButton gotoClarionGuiButton = new JButton();
  JButton outputFileButton = new JButton();
  JButton runButton = new JButton();
  JButton exitButton = new JButton();
  JButton confirmStep1Button = new JButton();
  JButton confirmStep2Button = new JButton();
  JButton confirmStep3Button = new JButton();
  JButton confirmStep4Button = new JButton();
  JButton confirmStep5Button = new JButton();
  JButton confirmStep6Button = new JButton();
  JButton gotoStep1Button = new JButton();
  JScrollPane jScrollPane1 = new JScrollPane( instructTextArea );

  /** constructor.
   *  @param cExp the main class.
   *  @param task the task to simulate. */
  public NewSimulator( ClarionMain cExp, SimuTask task )
  {
    try {
      mainFrame = cExp;
      taskName = task.getTaskName();
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception
  {
    thisFrame = this;
    this.setSize(new Dimension(884, 600) );
    this.getContentPane().setLayout(null);
    titilePanel.setBounds(new Rectangle(-1, 0, 708, 67));
    titilePanel.setLayout(gridLayout1);
    taskLabel.setFont(new java.awt.Font("Serif", 1, 24));
    taskLabel.setHorizontalAlignment(SwingConstants.CENTER);
    taskLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    taskLabel.setText( taskName );
    gridLayout1.setRows(2);
    gridLayout1.setColumns(1);
    copyRightLabel.setText("( Do not distribute or duplicate. All rights reserved. )");
    copyRightLabel.setHorizontalTextPosition(SwingConstants.CENTER);
    copyRightLabel.setHorizontalAlignment(SwingConstants.CENTER);
    copyRightLabel.setFont(new java.awt.Font("Serif", 1, 16));

    instructTextArea.setEditable(false);
    instructTextArea.setText( InstrText.getMainInstruction() );


    stepPanel.setBounds(new Rectangle(414, 72, 458, 486));
    stepPanel.setLayout(null);

    //step1Label.setFont(new java.awt.Font("Serif", 0, 16));
    step1Label.setMaximumSize(new Dimension(280, 15));
    step1Label.setMinimumSize(new Dimension(268, 15));
    step1Label.setPreferredSize(new Dimension(272, 15));
    step1Label.setText("Step 1 Set number of running agent simultaneously (1 to 100)");
    step1Label.setBounds(new Rectangle(28, 13, 306, 20));
    agentNumberTextField.setText("1");
    agentNumberTextField.setBounds(new Rectangle(64, 38, 30, 16));
    agentNumberTextField.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        agentNumberTextField_actionPerformed(e);
      }
    });

    step2Label.setEnabled(false);
    step2Label.setText("Step 2 : To be contorlled by :");
    step2Label.setBounds(new Rectangle(32, 78, 158, 22));
    controlRadioButton1.setEnabled(false);
    controlRadioButton1.setText("BY HUMANS");
    controlRadioButton1.setBounds(new Rectangle(71, 104, 101, 25));
    controlRadioButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        controlRadioButton1_actionPerformed(e);
      }
    });
    controlRadioButton2.setEnabled(false);
    controlRadioButton2.setSelected(true);
    controlRadioButton2.setText("BY CLARION");
    controlRadioButton2.setBounds(new Rectangle(182, 102, 106, 29));
    controlRadioButton2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        controlRadioButton2_actionPerformed(e);
      }
    });
    controlRadioButton3.setEnabled(false);
    controlRadioButton3.setText("BY ACT-R");
    controlRadioButton3.setBounds(new Rectangle(290, 101, 86, 28));
    controlRadioButton3.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        controlRadioButton3_actionPerformed(e);
      }
    });

    step3Label.setEnabled(false);
    step3Label.setText("Step 3 : Choose Configuration Mode");
    step3Label.setBounds(new Rectangle(31, 137, 193, 22));
    configRadioButton1.setEnabled(false);
    configRadioButton1.setSelected(true);
    configRadioButton1.setText("Configure with GUI");
    configRadioButton1.setBounds(new Rectangle(71, 168, 122, 17));
    configRadioButton1.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        configRadioButton1_actionPerformed(e);
      }
    });
    configRadioButton2.setEnabled(false);
    configRadioButton2.setText("Configure without GUI");
    configRadioButton2.setBounds(new Rectangle(212, 163, 149, 25));
    configRadioButton2.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        configRadioButton2_actionPerformed(e);
      }
    });

    step4Label.setEnabled(false);
    step4Label.setText("Step 4:");
    step4Label.setBounds(new Rectangle(29, 206, 40, 22));
    gotoTaskSpecificGuiButton.setText("Go To Task Specific GUI");
    gotoTaskSpecificGuiButton.setBounds(new Rectangle(83, 205, 174, 23));
    gotoTaskSpecificGuiButton.setEnabled(false);
    gotoTaskSpecificGuiButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        gotoTaskSpecificGuiButton_actionPerformed(e);
      }
    });

    step5Label.setEnabled(false);
    step5Label.setText("Step 5: ");
    step5Label.setBounds(new Rectangle(29, 258, 44, 22));
    gotoClarionGuiButton.setText("Go To CLARION GUI");
    gotoClarionGuiButton.setBounds(new Rectangle(84, 258, 173, 23));
    gotoClarionGuiButton.setEnabled(false);
    gotoClarionGuiButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        gotoClarionGuiButton_actionPerformed(e);
      }
    });

    step6Label.setEnabled(false);
    step6Label.setText("Step 6 : ");
    step6Label.setBounds(new Rectangle(27, 317, 46, 22));
    outputFileButton.setText("Select   Output   File");
    outputFileButton.setBounds(new Rectangle(83, 318, 175, 23));
    outputFileButton.setEnabled(false);
    outputFileButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        outputFileButton_actionPerformed(e);
      }
    });
    outputFileTextField.setEnabled(false);
    outputFileTextField.setToolTipText("");
    outputFileTextField.setEditable(false);
    outputFileTextField.setText("Input data file name");
    outputFileTextField.setBounds(new Rectangle(86, 359, 173, 24));
    outputFileTextField.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        outputFileTextField_actionPerformed(e);
      }
    });

    step7Label.setEnabled(false);
    step7Label.setText("Step 7 :");
    step7Label.setBounds(new Rectangle(27, 423, 44, 22));
    runButton.setEnabled(false);
    runButton.setText("RUN   TASK");
    runButton.setBounds(new Rectangle(76, 423, 117, 24));
    runButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        runButton_actionPerformed(e);
      }
    });
    exitButton.setText("EXIT");
    exitButton.setBounds(new Rectangle(206, 422, 112, 25));
    exitButton.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        exitButton_actionPerformed(e);
      }
    });


    confirmStep1Button.setBounds(new Rectangle(361, 16, 76, 22));
    confirmStep1Button.setText("confirm");
    confirmStep1Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep1Button_actionPerformed(e);
      }
    });
    confirmStep2Button.setText("confirm");
    confirmStep2Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep2Button_actionPerformed(e);
      }
    });
    confirmStep2Button.setBounds(new Rectangle(363, 74, 76, 22));
    confirmStep2Button.setEnabled(false);
    confirmStep3Button.setBounds(new Rectangle(364, 137, 76, 22));
    confirmStep3Button.setEnabled(false);
    confirmStep3Button.setText("confirm");
    confirmStep3Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep3Button_actionPerformed(e);
      }
    });
    confirmStep4Button.setBounds(new Rectangle(367, 201, 74, 22));
    confirmStep4Button.setEnabled(false);
    confirmStep4Button.setText("confirm");
    confirmStep4Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep4Button_actionPerformed(e);
      }
    });
    confirmStep5Button.setBounds(new Rectangle(367, 250, 76, 22));
    confirmStep5Button.setEnabled(false);
    confirmStep5Button.setText("confirm");
    confirmStep5Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep5Button_actionPerformed(e);
      }
    });
    confirmStep6Button.setBounds(new Rectangle(366, 312, 76, 22));
    confirmStep6Button.setEnabled(false);
    confirmStep6Button.setText("confirm");
    confirmStep6Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        confirmStep6Button_actionPerformed(e);
      }
    });
    gotoStep1Button.setBounds(new Rectangle(331, 422, 108, 25));
    gotoStep1Button.setEnabled(false);
    gotoStep1Button.setMaximumSize(new Dimension(120, 25));
    gotoStep1Button.setMinimumSize(new Dimension(100, 25));
    gotoStep1Button.setPreferredSize(new Dimension(104, 25));
    gotoStep1Button.setMnemonic('0');
    gotoStep1Button.setText("Go To Step 1");
    gotoStep1Button.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e) {
        gotoStep1Button_actionPerformed(e);
      }
    });
    jScrollPane1.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
    jScrollPane1.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
    jScrollPane1.setBounds(new Rectangle(6, 86, 405, 444));
    stepPanel.add(step2Label, null);
    stepPanel.add(step3Label, null);
    stepPanel.add(gotoTaskSpecificGuiButton, null);
    stepPanel.add(confirmStep4Button, null);
    stepPanel.add(confirmStep5Button, null);
    stepPanel.add(gotoClarionGuiButton, null);
    stepPanel.add(controlRadioButton2, null);
    stepPanel.add(controlRadioButton3, null);
    stepPanel.add(controlRadioButton1, null);
    stepPanel.add(configRadioButton1, null);
    stepPanel.add(configRadioButton2, null);
    stepPanel.add(outputFileTextField, null);
    stepPanel.add(step7Label, null);
    stepPanel.add(outputFileButton, null);
    stepPanel.add(confirmStep6Button, null);
    stepPanel.add(step6Label, null);
    stepPanel.add(step5Label, null);
    stepPanel.add(step4Label, null);
    stepPanel.add(runButton, null);
    stepPanel.add(exitButton, null);
    stepPanel.add(gotoStep1Button, null);
    stepPanel.add(step1Label, null);
    stepPanel.add(confirmStep1Button, null);
    stepPanel.add(agentNumberTextField, null);
    stepPanel.add(confirmStep2Button, null);
    stepPanel.add(confirmStep3Button, null);
    this.getContentPane().add(titilePanel, null);
    titilePanel.add(taskLabel, null);
    titilePanel.add(copyRightLabel, null);
    controlButtonGroup.add(controlRadioButton1);
    controlButtonGroup.add(controlRadioButton2);
    controlButtonGroup.add(controlRadioButton3);
    configButtonGroup.add(configRadioButton1);
    configButtonGroup.add(configRadioButton2);
    this.getContentPane().add(jScrollPane1, null);
    jScrollPane1.getViewport().add(instructTextArea, null);
    this.getContentPane().add(stepPanel, null);

    for( int i = 0; i < stepPanel.getComponentCount(); i++ )
         stepPanel.getComponent(i).setFont(new java.awt.Font("Serif", 0, 12));
  }

  /** event listener to the text field for receiving input of number of running
   *  agents.  */
  void agentNumberTextField_actionPerformed(ActionEvent e)
  {
      try{
        int num = Integer.parseInt( String.valueOf(agentNumberTextField.getText().trim()) );
        if( num < 0 || num > MAX_RUNNING_AGENT_NUM ){
            JOptionPane.showMessageDialog( null, "Out of range, reset !",
                                        "Error", JOptionPane.ERROR_MESSAGE );
            agentNumberTextField.setText( String.valueOf( runningAgentNum ));
        }
        else
            runningAgentNum = num;
      }
      catch( NumberFormatException e1 ){
         JOptionPane.showMessageDialog( null, "Not a number, reset !",
                                        "Error", JOptionPane.ERROR_MESSAGE );
         agentNumberTextField.setText( String.valueOf( runningAgentNum ) );
      }
  }

  /** event listener to button for choosing control mode.  */
  void controlRadioButton1_actionPerformed(ActionEvent e) {
     controlMode = BY_HUMAN;
  }

  /** event listener to button for choosing control mode.  */
  void controlRadioButton2_actionPerformed(ActionEvent e) {
     controlMode = BY_CLARION;
  }

  /** event listener to button for choosing control mode.  */
  void controlRadioButton3_actionPerformed(ActionEvent e) {
     controlMode = BY_ACTR;
  }

  /** event listener to button for choosing configuration mode.  */
  void configRadioButton1_actionPerformed(ActionEvent e)
  {
     configMode = W_GUI;
     Global.setSettingMode( configMode == W_GUI );
  }

  /** event listener to button for choosing configuration mode.  */
  void configRadioButton2_actionPerformed(ActionEvent e)
  {
     configMode = O_GUI;
     Global.setSettingMode( configMode == W_GUI );
  }

  /** event listener to button for configuration of task-spcific options.  */
  void gotoTaskSpecificGuiButton_actionPerformed(ActionEvent e)
  {
     //if( guiTask == null )    // initialize clarion option GUI.
         guiTask = new GuiTask( Global.TASK_SUBSYS_NUM, globals );
     guiTask.setMainFrame( thisFrame );
     guiTask.setRunningAgentNum( runningAgentNum );
     guiTask.setCurAgentIdx( 0 );
     guiTask.init( thisFrame );
     thisFrame.setEnabled( false );
     thisFrame.setVisible( false );
     guiTask.setEnabled( true );
     guiTask.setVisible( true );
  }

  /** event listener to button for configuration of CLARION options.  */
  void gotoClarionGuiButton_actionPerformed(ActionEvent e)
  {
     //if( guiClarion == null )    // initialize clarion option GUI.
         guiClarion = new GuiClarion( Global.CLARION_SUBSYS_NUM, globals );
     guiClarion.setMainFrame( thisFrame );
     guiClarion.setRunningAgentNum( runningAgentNum );
     guiClarion.setCurAgentIdx( 0 );
     guiClarion.init( thisFrame );
     thisFrame.setEnabled( false );
     thisFrame.setVisible( false );
     guiClarion.setEnabled( true );
     guiClarion.setVisible( true );
  }

  /** event listener to button for choosing output file.  */
  void outputFileButton_actionPerformed(ActionEvent e)
  {
       if ( wDataFile.openFile() != null ){
            outputFileTextField.setText( wDataFile.getFullFileName() );
            outputFileTextField.setEnabled( false );
            outputFileTextField.setEditable( false );
            confirmStep6Button.setEnabled( true );
       }
   }

  /** event listener to text field for choosing output file.  */
  void outputFileTextField_actionPerformed(ActionEvent e)
  {
      if( outputFileTextField.getText() == "" ){
         JOptionPane.showMessageDialog( this, "Please select a file",
                                "WARNING", JOptionPane.WARNING_MESSAGE );
         enterFlag = false;
      }
      else {
         String dataFileName = outputFileTextField.getText();
         File file = new File( dataFileName );
         if( !file.exists() ){
             JOptionPane.showMessageDialog( null, "File does not exits",
                  "Invalid File Name", JOptionPane.ERROR_MESSAGE );
             enterFlag = false;
         }
         else{
              enterFlag = true;
              wDataFile.setFile( file );
              //System.out.println( dataFile );
         }
      }
      confirmStep6Button.setEnabled( enterFlag );
  }

  /** the handler for processing the 'RUN' button. */
  void runButton_actionPerformed(ActionEvent e)
  {
      // open the data file to store the simulation data.
      //dataFile = wDataFile.openFile();
      if( JOptionPane.showConfirmDialog( null, "All options should be unchanged during task running, start a new task?",
          "WARNING", JOptionPane.YES_NO_OPTION ) == JOptionPane.NO_OPTION )
          return;

      if( configMode == W_GUI ){  // with GUI mode.
          System.out.println( "run with GUI !" );
          for( int i = 0; i < runningAgentNum; i++ )
              if (!guiTask.isConfirmed(Global.TASK_OPTIONS, i) || !guiTask.isConfirmed(Global.CLARION_OPTIONS, i) ) {
                String str = "Settings of the agent - " + String.valueOf(i) + " haven't been initialized, stop running!";
                JOptionPane.showMessageDialog(null, str, "WARNING", JOptionPane.WARNING_MESSAGE);
                return;
          }
          Global.getTask().setRunningAgentNum( runningAgentNum );
          Global.getTask().setGlobals( globals );
          Global.getTask().setDataFile( wDataFile );

          //System.out.println( "In NewSimulator.java, idn input sum 1 = " + globals[0].getIdnInputNum( 0 ));
          //System.out.println( "In NewSimulator.java, idn input sum 2 = " + globals[1].getIdnInputNum( 0 ));

          Global.getTask().start();
          if( guiClarion != null )
              guiClarion.setCurAgentIdx( 0 );
          if( guiTask != null )
              guiTask.setCurAgentIdx( 0 );
      }
      else{   // without GUI mode.
          Global.getTask().setRunningAgentNum( runningAgentNum );
          Global.getTask().setGlobals( globals );
          Global.getTask().setDataFile( wDataFile );
          Global.getTask().init();
          Global.getTask().start();
          if( guiClarion != null )
              guiClarion.setCurAgentIdx( 0 );
          if( guiTask != null )
              guiTask.setCurAgentIdx( 0 );
      }
  }

  /** event listener to button for exit the system.  */
  void exitButton_actionPerformed(ActionEvent e)
  {
      if( wDataFile != null )
          wDataFile.closeFile();
      System.exit( 0 );
  }

  /** event listener to button for confirm choice of step 1.  */
  void confirmStep1Button_actionPerformed(ActionEvent e)
  {
    agentNumberTextField_actionPerformed( e );
    for( int i = 0; i < runningAgentNum; i++ ){
         try{
           if( taskClarions[i] == null )
               taskClarions[i] = (TaskClarion)(taskClarionClass.newInstance());
         }
         catch( Exception e2 ){
                System.out.println( e2 );
         }
         if( globals[i] == null )
             globals[i] = new Global();
         taskClarions[i].setGlobal( globals[i] );
         globals[i].setTaskClarion( taskClarions[i] );
    }

     this.step1Label.setEnabled( false );
     this.agentNumberTextField.setEnabled( false );
     this.agentNumberTextField.setEditable( false );
     this.confirmStep1Button.setEnabled( false );
     this.step2Label.setEnabled( true );
     this.controlRadioButton1.setEnabled( true );
     this.controlRadioButton2.setEnabled( true );
     this.controlRadioButton3.setEnabled( true );
     this.confirmStep2Button.setEnabled( true );
  }

  /** event listener to button for confirm choice of step 2.  */
  void confirmStep2Button_actionPerformed(ActionEvent e)
  {
    this.step2Label.setEnabled( false );
    this.controlRadioButton1.setEnabled( false );
    this.controlRadioButton2.setEnabled( false );
    this.controlRadioButton3.setEnabled( false );
    this.confirmStep2Button.setEnabled( false );
    this.step3Label.setEnabled( true );
    this.configRadioButton1.setEnabled( true );
    this.configRadioButton2.setEnabled( true );
    this.confirmStep3Button.setEnabled( true );
  }

  /** event listener to button for confirm choice of step 3.  */
  void confirmStep3Button_actionPerformed(ActionEvent e)
  {
    this.step3Label.setEnabled( false );
    this.configRadioButton1.setEnabled( false );
    this.configRadioButton2.setEnabled( false );
    this.confirmStep3Button.setEnabled( false );
    if( configMode == W_GUI){
        this.step4Label.setEnabled(true);
        this.gotoTaskSpecificGuiButton.setEnabled(true);
        this.confirmStep4Button.setEnabled( true );
    }
    else {
        this.step6Label.setEnabled( true );
        this.outputFileButton.setEnabled( true );
        this.outputFileTextField.setEnabled( true );
        this.outputFileTextField.setEditable( true );
    }
  }

  /** event listener to button for confirm choice of step 4.  */
  void confirmStep4Button_actionPerformed(ActionEvent e)
  {
    if( guiTask == null ){
       JOptionPane.showMessageDialog(this, "Please finish configuration first!",
                                    "WARNING", JOptionPane.WARNING_MESSAGE);
       return;
    }

    for( int i = 0; i < this.runningAgentNum; i++ ){
      if (!(guiTask.isConfirmed(Global.TASK_OPTIONS, i))){
         JOptionPane.showMessageDialog( this, "Please finish configuration first!",
                                        "WARNING", JOptionPane.WARNING_MESSAGE );
         return;
      }
    }
    this.step4Label.setEnabled( false );
    this.gotoTaskSpecificGuiButton.setEnabled( false );
    this.confirmStep4Button.setEnabled( false );
    if( controlMode == BY_CLARION){
        this.step5Label.setEnabled(true);
        this.gotoClarionGuiButton.setEnabled(true);
        this.confirmStep5Button.setEnabled( true );
    }
    else{
        this.step6Label.setEnabled( true );
        this.outputFileButton.setEnabled( true );
        this.outputFileTextField.setEnabled( true );
        this.outputFileTextField.setEditable( true );
    }
  }

  /** event listener to button for confirm choice of step 5.  */
  void confirmStep5Button_actionPerformed(ActionEvent e)
  {
    if( guiClarion == null ){
       JOptionPane.showMessageDialog(this, "Please finish configuration first!",
                                    "WARNING", JOptionPane.WARNING_MESSAGE);
       return;
    }

    for( int i = 0; i < this.runningAgentNum; i++ ){
       if(!(guiClarion.isConfirmed(Global.CLARION_OPTIONS, i))){
           JOptionPane.showMessageDialog( this, "Please finish configuration first!",
                                          "WARNING", JOptionPane.WARNING_MESSAGE );
           return;
       }
    }

    this.step5Label.setEnabled( false );
    this.gotoClarionGuiButton.setEnabled( false );
    this.confirmStep5Button.setEnabled( false );
    this.step6Label.setEnabled( true );
    this.outputFileButton.setEnabled( true );
    this.outputFileTextField.setEnabled( true );
    this.outputFileTextField.setEditable( true );
  }

  /** event listener to button for confirm choice of step 6.  */
  void confirmStep6Button_actionPerformed(ActionEvent e)
  {
    this.step6Label.setEnabled( false );
    this.outputFileButton.setEnabled( false );
    this.outputFileTextField.setEnabled( false );
    this.outputFileTextField.setEditable( false );
    this.confirmStep6Button.setEnabled( false );
    this.step7Label.setEnabled( true );
    this.runButton.setEnabled( true );
    this.gotoStep1Button.setEnabled( true );
  }

  /** event listener to button for confirm choice of step 7.  */
  void gotoStep1Button_actionPerformed(ActionEvent e)
  {
    this.step7Label.setEnabled( false );
    this.runButton.setEnabled( false );
    this.gotoStep1Button.setEnabled( false );
    this.step1Label.setEnabled( true );
    this.agentNumberTextField.setEnabled( true );
    this.agentNumberTextField.setEditable( true );
    this.confirmStep1Button.setEnabled( true );
  }
}
