package tasksimulators;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import clarion.common.*;
import clarion.system.*;
import clarion.tools.*;
import main.ClarionMain;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to prepare configuration of the CLARION model. It is applied to
 * both old and new version of simuation.
 * <li> summary of the important methods. <br>
 * <ol>
 * </ol>
 * </ul>
 * <li> Role in the CLARION model <br>
 * This class is the generic class for simulation. All of the concrete human task
 * simulators should may extend this class for specific experimental process of
 * the task.
 *
 * control the simulation process of a specific human task.
 * <li> Usage of this class. <br>
 * It is instantiated inside a <em>ClarionMain</em> object.
 * </ul>
 * @author Xi Zhang
 *
 */

public class Simulator extends JFrame {
   final int WINDOW_WIDTH = 840,  WINDOW_HEIGHT = 600;
   final int MIN_BLOCK_NUM = 1, MAX_BLOCK_NUM = 1000, MAX_SIMU_MODE_NUM = 3;
   protected String SIMU_PARA_SET_TEXT[] = {
       "CLARION  MODEL", "ACT-R  MODEL"
   };
   protected String MODE_BUTTON_TEXT[] = {
       "BY HUMANS", "BY CLARION", "BY ACT-R"
   };
   protected boolean enterFlag;
   protected int expMode, blockNum;
   protected String dataFileName;
   protected WriteFile wDataFile;
   protected RandomAccessFile dataFile;
   protected Container c;
   protected JPanel inPanel;
   protected JTextField inputBlock, inputFileField;
   protected TextArea instrTextArea;
   protected JButton openButton, enterButton, exitButton, simuParaSetButtons[];
   protected JRadioButton simuModeButtons[];
   protected ClarionMain exp;
   protected SimuTask simuTask;
   //initialize an instance of the class for getting instructions
   InstrText instrText = new InstrText();

   public Simulator( ClarionMain cExp, SimuTask task )
   {
      exp = cExp;
      simuTask = task;
      wDataFile = new WriteFile();
      setSize( WINDOW_WIDTH, WINDOW_HEIGHT );
      c = getContentPane();
      c.removeAll();
      c.setLayout( new BorderLayout( ) );
      // north area.
      JPanel cPanel = new JPanel( new BorderLayout() );
      c.add( cPanel, BorderLayout.NORTH );
      // caption.
      JLabel clarionCaption = new JLabel( simuTask.getTaskName() );
      clarionCaption.setFont( new Font( "Serif", Font.BOLD, 35 ) );
      JPanel capPanel = new JPanel( new FlowLayout(FlowLayout.CENTER, 5,5 ) );
      capPanel.add( clarionCaption );
      // right reserve.
      JLabel clarionRights  = new JLabel( instrText.getRightReserveInfo() );
      clarionRights.setFont(  new Font( "Serif", Font.BOLD, 14 ) );
      JPanel rgtPanel = new JPanel( new FlowLayout(FlowLayout.CENTER, 5,5 ) );
      rgtPanel.add( clarionRights );
      cPanel.add( capPanel, BorderLayout.CENTER );
      cPanel.add( rgtPanel, BorderLayout.SOUTH );
      // central area.
      JPanel conPanel = new JPanel( new GridLayout(1,2,20,10) );
      c.add( conPanel, BorderLayout.CENTER );
      // instructions.
      JPanel insPanel = new JPanel( new FlowLayout(FlowLayout.CENTER, 5, 5) );
      conPanel.add( insPanel );
      TextArea instrTextArea = new TextArea( instrText.getTaskInstructions(simuTask.getTaskName()),
                                             instrText.ROWS, instrText.COLS );
      instrTextArea.setFont( new Font( "Serif", Font.PLAIN, 15 ) );
      instrTextArea.setEditable( false );
      insPanel.add( instrTextArea );
      // user area.
      JPanel userPanel = new JPanel( new GridLayout( 3, 1, 10, 10 ) );
      // input area, should be overrided
      //inPanel = new JPanel( new FlowLayout( FlowLayout.CENTER, 200, 10 ) );
      inPanel = new JPanel();
      inPanel.setLayout( null );
      // simulation buttons.
      JPanel sPanel = new JPanel( new GridLayout( MAX_SIMU_MODE_NUM + 1, 1, 5, 5) );
      JPanel simPanel[] = new JPanel[ MAX_SIMU_MODE_NUM + 1 ];
      simPanel[0] = new JPanel( new FlowLayout(FlowLayout.LEFT) );
      JLabel simLabel = new JLabel( "To be controlled by: " );
      simLabel.setFont( new Font( "Serif", Font.BOLD, 20 ) );
      simPanel[0].add( simLabel );
      sPanel.add(simPanel[0]);
      for( int i = 1; i < MAX_SIMU_MODE_NUM+1; i++ ){
           simPanel[i] = new JPanel();
           simPanel[i].setLayout( new FlowLayout(FlowLayout.LEFT, 20, 5) );
      }
      SimuModeButtonHandler handler = new SimuModeButtonHandler();
      ButtonGroup modeGroup = new ButtonGroup();
      simuModeButtons = new JRadioButton[ MAX_SIMU_MODE_NUM ];
      simuParaSetButtons  = new JButton[ MAX_SIMU_MODE_NUM - 1 ];
      for( int i = 0; i < MAX_SIMU_MODE_NUM; i++ ){
         if( i == expMode )
             simuModeButtons[ i ] = new JRadioButton( MODE_BUTTON_TEXT[i], true );
         else
             simuModeButtons[ i ] = new JRadioButton( MODE_BUTTON_TEXT[i], false );
         simuModeButtons[ i ].addItemListener( handler );
         modeGroup.add( simuModeButtons[ i ] );
         simPanel[i+1].add( simuModeButtons[ i ] );
         if( i > 0 ){
             simuParaSetButtons[ i-1 ] = new JButton( SIMU_PARA_SET_TEXT[i-1] );
             simuParaSetButtons[ i-1 ].setEnabled( false );
             simPanel[i+1].add( simuParaSetButtons[ i-1 ] );
         }
         sPanel.add(simPanel[i+1]);
      }
      simuParaSetButtons[0].addActionListener(
                 new  ActionListener() {
                     public void actionPerformed( ActionEvent e ){
                        setEnabled( false );
                        setVisible( false );
                        exp.setEnabled(true);
                        exp.setVisible(true);
                     }
                 }   );
      // operation buttons.
      JPanel bPanel = new JPanel( );
      bPanel.setLayout( new FlowLayout(FlowLayout.LEFT, 10, 10) );
      // select file button.
      openButton = new JButton( "SELECT OUTPUT DATA FILE" );
      openButton.setEnabled( true );
      openButton.addActionListener(
                 new  ActionListener() {
                     public void actionPerformed( ActionEvent e )
                     {
                         enterFlag = selectFile();
                         enterButton.setEnabled( enterFlag );
                     }
                 }
      );
      bPanel.add( openButton );
      // run button.
      enterButton = new JButton( "RUN" );
      enterButton.setEnabled( enterFlag );
      enterButton.addActionListener( new  ActionListener() {
               public void actionPerformed( ActionEvent e ){
                   runButton_actionPerformed( e );
               }
          }   );
      bPanel.add( enterButton );
      // exit button.
      exitButton = new JButton( "EXIT" );
      exitButton.setEnabled( true );
      exitButton.addActionListener(
                 new  ActionListener() {
                     public void actionPerformed( ActionEvent e )
                     {
                        if( dataFile != null )
                            wDataFile.closeFile();
                        System.exit( 0 );
                     }
                 }
      );
      bPanel.add( exitButton );
      userPanel.add( inPanel );
      userPanel.add( sPanel );
      userPanel.add( bPanel );
      conPanel.add( userPanel );
      setEnabled( false );
      setVisible( false );
      validate();
   }

   public WriteFile getDataFile()
   {
      return wDataFile;
   }

   protected boolean checkInput()
   {
       return true;
   }

   protected boolean checkFile()
   {
       return true;
   }

   /**  the handler for processing the 'RUN' button. this method needs to be
    *   overrided. */
   protected void runButton_actionPerformed( ActionEvent e )
   {
       if( checkInput() == true && checkFile() == true ){
            setEnabled( false );
            setVisible( false );
            //World.TASK_PARA_ARRAY = exp.getTaskParaSet();
            simuTask.start( expMode, blockNum, wDataFile );
       }
   }

/********************************PRIVATE METHODS *******************************/

  // analyze user's inputs of block num and file name.
   private boolean selectFile()
   {
       dataFile = wDataFile.openFile();
       if ( dataFile != null ){
             dataFileName = wDataFile.getFullFileName();
             if( inputFileField != null )
                 inputFileField.setText( dataFileName );
             return true;
       }
       else{
            if( inputFileField == null )
                return true;
            if( inputFileField.getText() == "" ){
               JOptionPane.showMessageDialog( this, "Please select a file",
                                      "WARNING", JOptionPane.WARNING_MESSAGE );
                return false;
            }
            else
                return true;
      }
   }

   // experiment selecting( PC_SIMULATOR OR CLARION_MODEL, ACT-R_MODEL )
   // button action handler.
   private class SimuModeButtonHandler implements ItemListener
   {
      public void itemStateChanged( ItemEvent e )
      {
        for( int i = 0; i < MAX_SIMU_MODE_NUM; i++ )
           if( e.getSource() == simuModeButtons[ i ] )
           {
               expMode = i;
               if( i > 0 )
                   simuParaSetButtons[i-1].setEnabled( true );
           }
           else
           {
               if( i > 0 )
                   simuParaSetButtons[i-1].setEnabled( false );
           }
      }
   }
}
