package tasksimulators.newsimu.AGL;
import java.io.*;
import java.util.Stack;
import java.util.Vector;
import clarion.system.*;
import clarion.tools.*;

/**
 * <ul>
 * <li> General overview.
 * <ul>
 * <li> summary of this class <br>
 * This class is to simulate Artificial Grammarical Learning task. It extends the
 * <tt>Task</tt> class.
 * <li> important methods <br>
 * <ol>
 * <li> public void start() : the start point of a task to run. This method will
 * be called by the system automatically when you click the 'RUN TASK' button in
 * the task simulation screen. It implements the task-specific routine of the AGL
 * task.
 * <li> public void init() : the method overrides the one in <tt>Task</tt>, it
 * implements the process of initialization of <tt>Global</tt> objects assoicated
 * with the running agents by loading the existing setting groups from the hard
 * disk.
 * </ol>
 * </ul>
 * <li> Role in the CLARION model. <br>
 * This class implemnts the Artificial Grammarical task using CLARION and part of
 * AGL task simulator.
 * <li> Usage of this class. <br>
 *  Routines in this class will be called by the system when the AGL task is loaded
 *  for simulation.
 * </ul>
 * @author Xi Zhang
 *
 **/

public class TaskAGL extends Task implements SimuTask {

  /******************** VARIABLES FOR SPECIFIC TASK ****************************/
    //final int GROUP_NUM = 6, AGENT_NUM = 20, SESSION_NUM = 2;
    final int GROUP_NUM = 6, AGENT_NUM = 20, SESSION_NUM = 2, MEASURE_NUM = 5;
    final int MAX_STR_LEN = 11, MIN_STR_LEN = 5, SAMPLE_NUM = 177;
    final int INPUT_DIM_NUM = MAX_STR_LEN + 2, OUTPUT_DIM_NUM = MAX_STR_LEN + 1;
    final int INPUT_NUM = 83, OUTPUT_NUM = 89, ACTION_NUM = 7;
    final int LETTER_DIM_LEN = 7, LEARN_TYPE_NUM = 4, STAGE_NUM = 2, POSITION_NUM = MAX_STR_LEN + 1;
    final char[][] groupTypes = { {'C','C'}, {'I','E'}, {'E','I'}, {'I','I'}, {'E','E'}, {'S','S'} };
    final char[] letters = { '$', 'C', 'P', 'S', 'T', 'V', 'X', '-' };
    final String[] oldGroupNames = { "CC", "IE", "EI", "II", "EE", "SS" };
    final String[] newGroupNames = { "C,C", "ExP,GR", "GR,ExP", "ExP,ExP", "GR,GR", "ExD,ExD" };
    final String[] measureNames = { "achievement", "accuracy", "efficiency", "speed", "attempts"};
    boolean finishFlag;
    //int agentIdx, groupIdx = 0;
    int agentIdx, groupIdx = 0;
    // the 177 strings with length range from 5 to 11.
    String[] dataStrings;
    short[][][] dataBase;
    String[] trainStrings;
    short[][][] trainSet;
    String[] testStrings;
    short[][][] testSet;
    //short[][][] attempts;
    //double[][][] achievements, accuracies, efficiencies, speeds;
    double[][][][] wholeModelData;
    WriteFile wholeModelDataFiles[][], agentDataFiles[];


  //final int IN_LETTER = 0, IN_GROUP = 1, IN_STAGE = 2;
  //final int OUT_STAGE = 0, OUT_TRAIN = 1, OUT_TEST = 2, OUT_POS = 3;
  final int L_CON = 0, L_IMP = 1, L_EMP = 2, L_SYN = 3;
  final int S_TRAIN = 0, S_TEST = 1;
 /******************** VARIABLES FOR CLARION ****************************/
  int    featureTypeNum = 2;
  short  featureNums[] = { INPUT_DIM_NUM, OUTPUT_DIM_NUM };
  short  featureLengths[][]; {
     // letters, learning type(C,I,E,S), stages(train or test), train GKS or AMN, test using GKS or AMN
     featureLengths = new short[2][];
     int length = featureNums[0];
     featureLengths[0] = new short[length];
     for( int i = 0; i < length - 2; i++ )
          featureLengths[0][i] = LETTER_DIM_LEN;
     featureLengths[0][length-2] = LEARN_TYPE_NUM;
     featureLengths[0][length-1] = STAGE_NUM;
     length = featureNums[1];
     featureLengths[1] = new short[length];
     for( int i = 0; i < length - 1; i++ )
          featureLengths[1][i] = LETTER_DIM_LEN;
     featureLengths[1][length-1] = POSITION_NUM;
  }
  String featureNames[][]; {
     featureNames = new String[2][];
     int length = featureNums[0];
     featureNames[0] = new String[length];
     for( int i = 0; i < length - 2; i++ )
          featureNames[0][i] = "letter" + i;
     featureNames[0][length-2] =  "learning type";
     featureNames[0][length-1] = "stage";
     length = featureNums[1];
     featureNames[1] = new String[length];
     for( int i = 0; i < length - 1; i++ )
          featureNames[1][i] = "letter" + i;
     featureNames[1][length-1] = "position";
  }
  String featureValueNames[][][]; {
    String[] letterNames = {"$", "C", "P", "S", "T", "V", "X"};
    String[] learningNames = {"CON", "IMP", "EMP", "SYN" };
    String[] stageNames =  {"train", "test"};
    String[] positionNames = {"begin", "pos_1", "pos_2","pos_3","pos_4","pos_5","pos_6",
                            "pos_7","pos_8","pos_9","pos_10","pos_11"};
    featureValueNames = new String[2][][];
    int length = featureNums[0];
    featureValueNames[0] = new String[length][];
    for( int i = 0; i < length - 2; i++ )
         featureValueNames[0][i] = CommonMethods.copy( letterNames );
    featureValueNames[0][length-2] = CommonMethods.copy( learningNames );
    featureValueNames[0][length-1] = CommonMethods.copy( stageNames );
    length = featureNums[1];
    featureValueNames[1] = new String[length][];
    for( int i = 0; i < length - 1; i++ )
         featureValueNames[1][i] = CommonMethods.copy( letterNames );
    featureValueNames[1][length-1] = CommonMethods.copy( positionNames );
  }

 /******************** SETTINGS ABOUT ACS COMPONENTS **************************/
  int     acsNetNum = 1;
  int     gsOption = Global.GS_NON;
  boolean wmOn = false;
  int     exNetNum = 1;
 /************************** SETTINGS ABOUT IDN *******************************/
  int     exIdnNum = 1;
  short[] idnInputFeatureInfo[][];
  {
    short[] order = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12 };
    idnInputFeatureInfo = new short[ 1 ][ INPUT_DIM_NUM ][ 2 ];
    for( int i = 0; i < INPUT_DIM_NUM; i++ ){
         idnInputFeatureInfo[0][i][0] = (short)Global.FT_INPUT;
         idnInputFeatureInfo[0][i][1] = order[i];
    }
  }
  short[] idnOutputFeatureInfo[][];
  {
    short[] order = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
    idnOutputFeatureInfo = new short[ 1 ][ OUTPUT_DIM_NUM ][ 2 ];
    for( int i = 0; i < OUTPUT_DIM_NUM; i++ ){
         idnOutputFeatureInfo[0][i][0] = (short)Global.FT_OUTPUT;
         idnOutputFeatureInfo[0][i][1] = order[i];
    }
  }
  short[] idnInputDimVals[];
  short[] idnOutputDimVals[];
  {
    idnInputDimVals = new short[ 1 ][];
    idnInputDimVals[0] = CommonMethods.copy( featureLengths[0] );
    idnOutputDimVals = new short[ 1 ][];
    idnOutputDimVals[0] = CommonMethods.copy( featureLengths[1] );
  }
  short[] idnHiddenNums = { 30 };
  short[] idnBLLearningOptions = { Global.SQL };
  double  idnQDiscountRates[] = {0.95};
  double  idnBPLRs[] = { 0.3 };
  double  idnBPMoms[] = { 0.1 };
  double  idnActionSelTemps[] = { 0.1 };
  double  overallActionSelTemps[] = { 0.1 };
 /************************** SETTINGS ABOUT ARS *******************************/
  short[] arInputNums[] = {{ INPUT_NUM, INPUT_NUM, INPUT_NUM, INPUT_NUM }};
  short[] arOutputNums[] = {{ OUTPUT_NUM, OUTPUT_NUM, OUTPUT_NUM, OUTPUT_NUM }};
  short[] arInputDimVals[][] = {{ idnInputDimVals[0], idnInputDimVals[0],
                                  idnInputDimVals[0], idnInputDimVals[0]} };
  short[] arOutputDimVals[][] ={{ idnOutputDimVals[0], idnOutputDimVals[0],
                                  idnOutputDimVals[0], idnOutputDimVals[0]} };
  double  arActionSelTemps[][] = {{ 0.1, 0.1, 0.1, 0.1 }};
  short[][] arReasoningMethods = {{ Global.FORWARD_CHAIN, Global.FORWARD_CHAIN,
                                    Global.FORWARD_CHAIN, Global.FORWARD_CHAIN}};
  boolean arReasoningByMCSOns[][];
 /********************* SETTINGS ABOUT RULE LEARNING **************************/
  int     acsKnowledgeNum = 4;
  boolean rerOns[] = { false };
  boolean irlOns[] = { false };
  //boolean frOns[] = { true };
  boolean frOns[] = { true };
  boolean extInstructOns[] = { false };
  boolean ilOns[] = { false };
  boolean acsRLByMCSs[] = { false };
  short[] frNums = { ACTION_NUM };
  boolean frActionVariedOns[][];
  {
     frActionVariedOns = new boolean[ 1 ][ ACTION_NUM ];
     for( int i = 0; i < ACTION_NUM; i++ )
          frActionVariedOns[0][i] = false;
  }
  short[] frConditions[][][];
  //short[] frActions[][] = {{{0,0,0},{0,1,0},{0,2,0},{0,3,0},{1,0,1},{1,0,2},{1,0,3}}};
  short[] frActions[][][][];
  {  // initialize FR rules.
     frConditions = new short[1][ACTION_NUM][INPUT_DIM_NUM][];
     frActions = new short[1][ACTION_NUM][2][][];
     short[][][] actionConditions = {
       {{L_CON},{S_TRAIN}}, {{L_IMP},{S_TRAIN}}, {{L_EMP},{S_TRAIN}}, {{L_SYN},{S_TRAIN}},
       {{L_CON,L_IMP},{S_TEST}}, {{L_EMP},{S_TEST}}, {{L_SYN},{S_TEST}}
     };
     for( int i = 0; i < ACTION_NUM; i++ ){
       for( int j = 0; j < INPUT_DIM_NUM-2; j++ ){
            frConditions[0][i][j] = new short[ LETTER_DIM_LEN ];
            for( int k = 0; k < frConditions[0][i][j].length; k++ )
                 frConditions[0][i][j][k] = 1;
       }
       frConditions[0][i][INPUT_DIM_NUM-2] = new short[LEARN_TYPE_NUM];
       for( int j = 0; j < actionConditions[i][0].length; j++ )
            frConditions[0][i][INPUT_DIM_NUM-2][actionConditions[i][0][j]] = 1;
       frConditions[0][i][INPUT_DIM_NUM-1] = new short[STAGE_NUM];
       for( int j = 0; j < actionConditions[i][1].length; j++ )
            frConditions[0][i][INPUT_DIM_NUM-1][actionConditions[i][1][j]] = 1;
     }
  }

 /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
  boolean ruleUtilityOns[][][] = {{{},{},{},
  {false,false,false,false,false,false,false } }};
  boolean ruleSupportOns[][][] = {{{},{},{},
  {false,false,false,false,false,false,false } }};
  boolean partMatchOns[][][] = {{{},{},{},
  {false,false,false,false,false,false,false } }};
  boolean ruleBlaOns[][][] = {{{},{},{},
  {true,true,true,true,true,true,true } }};
  boolean actionChunkBlaOns[][][] = {{{},{},{},
  {true,true,true,true,true,true,true } }};
  short  actionRuleBlaOptions[][][] = {{{},{},{},{1,1,1,1,1,1,1}}};
  double actionRuleBlaParas[][][][] = {{ {{}}, {{}}, {{}},
    {{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0}}
  }};
  short  acsChunkBlaOptions[][][] = {{{},{},{},{1,1,1,1,1,1,1}}};
  double acsChunkBlaParas[][][][] = {{ {{}}, {{}}, {{}},
    {{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0},{2.0,0.5,0.0}}
  }};
 /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
  int    combineDeterMode = Global.COMB_NBY_MCS;
  short  crossCombModes[][] = { {Global.STOCH_COMB_MODE, Global.FIX_COMB_MODE} };
/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/
  double  crossCombineParas[][] = { {0.1, 0.0, 0.0, 0.9} };
 /******************** SETTINGS ABOUT NACS COMPONENTS *************************/
  boolean gksOnFlag = true;
  boolean emOnFlag = false;
  boolean aemOnFlag = false;
 /******************** SETTINGS ABOUT GKS REPRESENATIONS **********************/
  int     gksMaxChunkNum = 50;
  double  gksCSTemp = 0.1;
  boolean esChunkOnFlag = false;
  boolean similarityOn = false;
  int     assocConditionWeightMode = Global.CONSTANT_OPTION;
  int     assocDimensionWeightMode = Global.CONSTANT_OPTION;
  boolean assocBlaOn = true;
  int     assocBlaOption = Global.CONSTANT_OPTION;
  double  assocBla = 1.0;
  //double  assocBlaParas[] = {2.0,0.5,0.0};
  boolean nacsChunkBlaOn = true;
  int     chunkBlaOption = Global.CONSTANT_OPTION;;
  double  chunkBla = 1.0;
  //double  chunkBlaParas[] = {2.0,0.5,0.0};
  short[] emBlaOptions = { 0, 0 };
  double  emBlas[] = { 1.0, 1.0 };
  double  emBlaParas[][] = { {2.0,0.5,0.0}, {2.0,0.5,0.0} };
  double  threshold_EM = 0.3;
  int     emCapacity = 500;
  double  emTrainProb = 1.0;
  int     c9 = 10;        // # of EM items for training ACS.
  int     c10 = 2;        // # of iterations for training ACS.
  double  threshold_sim = 0.5;
  //double  c14 = 1.0;
  //double  c15 = 1.0;
 /*********************** SETTINGS ABOUT GKS LEARNINGS ************************/
  boolean assocExtractOn = false;
  boolean assocEncodeExternOn = true;
  //boolean assocAssimiOn = false;
  boolean assocAssimiOn = true;
  boolean chunkEncodeExternOn = true;
  boolean chunkFromAmnOn = false;
  boolean chunkByMCSOn = false;
  double  p_c = 1.0;
  double  p_a = 1.0;
  double  d_c = 0.0001;
  double  d_a = 0.0001;
  double  threshold_ae = 0.5;
  double  threshold_ce = 0.5;
  double  p_ae = 1.0;
  double  p_ce = 1.0;
 /******************** SETTINGS ABOUT AMN REPRESENATIONS **********************/
 /*-------- AMN SETTINGS --------*/
  int     amnNum = 1;
  short[] amnInputChunkTypes = { Global.CHK_STATE};
  short[] amnOutputChunkTypes = { Global.CHK_STATE};
  short[] amnInputFeatureInfo[][];
  short[] amnInputDimVals[];
  {
     amnInputDimVals = new short[ 1 ][INPUT_DIM_NUM - 2];
     amnInputFeatureInfo = new short[ 1 ][INPUT_DIM_NUM - 2][];
     for( int i = 0; i < INPUT_DIM_NUM - 2; i++ ){
          amnInputDimVals[0][i] = LETTER_DIM_LEN;
          amnInputFeatureInfo[0][i] = CommonMethods.copy(idnInputFeatureInfo[0][i]);
     }
  }
  short[] amnOutputFeatureInfo[][];
  short[] amnOutputDimVals[];
  {
     amnOutputDimVals = new short[ 1 ][INPUT_DIM_NUM - 2];
     amnOutputFeatureInfo = new short[ 1 ][INPUT_DIM_NUM - 2][];
     for( int i = 0; i < INPUT_DIM_NUM - 2; i++ ){
          amnOutputDimVals[0][i] = LETTER_DIM_LEN;
          amnOutputFeatureInfo[0][i] = CommonMethods.copy(idnOutputFeatureInfo[0][i]);
     }
  }
  short[] amnHiddenNums = { 30 };
  boolean amnMultiPassOns[] = { false };
  short   amnIterNums[] = { 1 };
 /*-------- HAM SETTINGS --------*/
  double  amnTrainProb = 1.0;
  int     amnTrainItemNum = 5;
  int     amnTrainIterNum = 5;
 /*-------- AMN BP SETTINGS --------*/
  double  amnLRs[] = { 0.1 };
  double  amnMoms[] = { 0.1 };
 /*-------- AEM SETTINGS --------*/
  int     aemStateNumBound = 5;
  double  aemTrainedByEmProb = 1.0;
  int     aemTrainedByEmItemNum = 30;
  int     aemTrainedByEmIterNum = 1;      // new option.
  double  aemTrainedByEmLR = 0.5;
  double  aemTrainedByEmMom = 0.1;
  int     aemNSNum = 5;
  double  aemTrainAcsProb = 1.0;
  int     aemTrainAcsSampleNum = 10;
  int     aemTrainAcsIterNum = 2;
  double  aemTrainAcsOutputTemp = 0.1;
/**************************** RT SETTINGS ************************************/
  boolean rtOn = true;
  int     maxUsageNum = 5000;
  int     PT_BL = 2000;
  int     DT_BL = 350;
  int     AT_BL = 500;
  int     PT_TL = PT_BL + 100;
  int     DT_TL_actionTime = 1200;
  double  t0 = 90;
  double  t1 = 90;
  //double  t0 = 200;
  //double  t1 = 200;
  int     AT_TL = 4000;
  double  t4 = 70;
  double  t5 = 70;
  double  t2 = 70;
  double  t3 = 70;
  int     t_AMN = 350;
/************************* TASK SPECIFIC **************************************/
  String taskName = "AGL";
  String rootPath = System.getProperty("user.dir");
  String optionFileRelativePath = "taskSetting";
  String dataFileRelativePath = "dataFiles";
  boolean[]  inputFormat = { true, true, true, false, false };
  boolean[]  outputFormat = { true, true, true, false, false };

  public TaskAGL()
  {
     //System.out.println( "in TaskAGL constructor " );

     //createDataFiles();
     createSamples();
     //attempts = new short[GROUP_NUM][AGENT_NUM][SESSION_NUM];
     //achivements = new double[GROUP_NUM][AGENT_NUM][SESSION_NUM];
     //accuracies = new double[GROUP_NUM][AGENT_NUM][SESSION_NUM];
     //efficiencies = new double[GROUP_NUM][AGENT_NUM][SESSION_NUM];
     //speeds = new double[GROUP_NUM][AGENT_NUM][SESSION_NUM];
     wholeModelData = new double[GROUP_NUM][AGENT_NUM][MEASURE_NUM][SESSION_NUM];
  }

  /** The method overrides the one in <tt>Task</tt>, it implements the process of
   * initialization of <tt>Global</tt> objects assoicated with the running agents
   * by loading the existing setting groups from the hard disk. */
  public void init()
  {
     //initClarion();    //revised on 03/07/2005.
     String[] names = { "subjectGroup0" };
     super.settingNames = names;
     super.init();
  }

  /** the start point of a task to run. This method will be called by the system
   * automatically when you click the 'RUN TASK' button in the task simulation
   * screen. It implements the task-specific routine of the AGL task. */
  public void start()
  {
     createDataFiles();
     groupIdx = 0;

     agents = new AgentAGL[1];
     initNewGroup();
     finishFlag = false;
     while( !finishFlag ){
        ((AgentAGL)agents[0]).run();
        //((AgentAGL)agents[0]).getAttempts( attempts[groupIdx][agentIdx] );
        //((AgentAGL)agents[0]).getAchievement( achievements[groupIdx][agentIdx] );
        //((AgentAGL)agents[0]).getAccuracy( accuracies[groupIdx][agentIdx] );
        //((AgentAGL)agents[0]).getEfficiency( efficiencies[groupIdx][agentIdx] );
        //((AgentAGL)agents[0]).getSpeed( speeds[groupIdx][agentIdx] );
        ((AgentAGL)agents[0]).getAchievement( wholeModelData[groupIdx][agentIdx][0] );
        ((AgentAGL)agents[0]).getAccuracy( wholeModelData[groupIdx][agentIdx][1] );
        ((AgentAGL)agents[0]).getEfficiency( wholeModelData[groupIdx][agentIdx][2] );
        ((AgentAGL)agents[0]).getSpeed( wholeModelData[groupIdx][agentIdx][3] );
        ((AgentAGL)agents[0]).getAttempts( wholeModelData[groupIdx][agentIdx][4] );
        gotoEndThisAgent();
     }
     System.out.println( "task is finished!" );
  }

  /** from <tt>SimuTask</tt> class. */
  public String getTaskName()
  {
     return "ARTIFICIAL GRAMMAR LEARNING TASK";
  }

  /** from <tt>SimuTask</tt>. */
  public void start( int expMode, int blockNum, WriteFile wFile )
  {
  }

/**************************PRIVATE METHODS*************************************/
  /** create the files to save data. */
  private void createDataFiles()
  {
     //String fileDirName = "dataFiles\\generic\\AGL\\";
     String fileDirName = modelDataFile.getCurDir() + "\\";
     //String modelFileName = "averageModelData";
     //modelDataFile.openFile( 1, fileDirName + modelFileName );
     wholeModelDataFiles = new WriteFile[SESSION_NUM][MEASURE_NUM];
     for( int i = 0; i < SESSION_NUM; i++ ){
          wholeModelDataFiles[i] = new WriteFile[MEASURE_NUM];
          for( int j = 0; j < MEASURE_NUM; j++ ){
               wholeModelDataFiles[i][j] = new WriteFile();
               wholeModelDataFiles[i][j].openFile( 1, fileDirName + "SESSION_" + (i+1) +
                                                  "_MSR_" + measureNames[j] );
          }
     }
     agentDataFiles = new WriteFile[GROUP_NUM];
     for( int i = 0; i < GROUP_NUM; i++ ){
          agentDataFiles[i] = new WriteFile();
          String fileName = "GRP_" + String.valueOf(groupTypes[i]);
          agentDataFiles[i].openFile( 1, fileDirName + fileName );
     }
  }

  /** generate the 177 test samples. */
  private void createSamples()
  {
     // create all of the sample strings.
     int TRAIN_SAMPLE_NUM = 22;
     dataStrings = new String[ SAMPLE_NUM ];
     trainStrings = new String[ TRAIN_SAMPLE_NUM ];
     testStrings = new String[ SAMPLE_NUM ];
     try{
        String DirName = "dataFiles\\generic\\AGL\\samples\\";
        RandomAccessFile souFile1, souFile2;
        souFile1 = new RandomAccessFile( DirName+"trainSamples24.txt", "r" );
        souFile2 = new RandomAccessFile( DirName+"testSamples.txt", "r" );
        souFile1.seek( 0 );
        souFile2.seek( 0 );
        for( int i = 0; i < TRAIN_SAMPLE_NUM; i++ )
             trainStrings[i] = souFile1.readLine();
        for( int i = 0; i < SAMPLE_NUM; i++ )
             testStrings[i] = souFile2.readLine();
        souFile1.close();
        souFile2.close();
        //createFullStrings();
     }
     catch ( IOException e ) {
             System.out.println( e );
     }
     // create all of sample sets.
     dataBase = new short[SAMPLE_NUM][][];
     trainSet = new short[TRAIN_SAMPLE_NUM][][];
     testSet = new short[SAMPLE_NUM][][];
     //createDataSet( dataStrings, dataBase );
     createDataSet( trainStrings, trainSet );
     createDataSet( testStrings, testSet );
   }

   /** create the data sets for constructing training and testing samples. */
   private void createDataSet( String[] dataStrs, short[][][] dataSet )
   {
     int sampleNum = dataSet.length;
     for( int i = 0; i < sampleNum; i++ ){
          int sampleLength = dataStrs[i].length();
          dataSet[i] = new short[ sampleLength ][ LETTER_DIM_LEN ];
          for( int j = 0; j < sampleLength; j++ )
            for( int k = 0; k < LETTER_DIM_LEN; k++ )
                 dataSet[i][j][k] = 0;
          for( int j = 0; j < sampleLength; j++ ){
               int k = 0;
               while( dataStrs[i].charAt(j) != letters[k] )
                      k++;
               if( k < LETTER_DIM_LEN )
                   dataSet[i][j][k] = 1;
          }
     }
   }

  // generate the 177 samples.
  private void createFullStrings()
  {
     char[][][] stateTrans = {
        {{'S','1'},{'C','2'}},  {{'C','3'}},            {{'V','1'},{'X','3'}},
        {{'T','4'},{'P','5'}},  {{'S','4'},{'X','6'}},  {{'T','5'},{'V','7'}},
        {{'S','8'},{'X','5'}},  {{'P','6'},{'V','8'}}
     };
     for( int i = 0; i < SAMPLE_NUM; i++ ){
          int length = testStrings[ i ].length();
          short[] filledLocs = new short[ 2 ];
          int count = 0;
          for( int j = 0; j < length; j++ )
               if( testStrings[i].charAt(j) != '-' )
                   filledLocs[count++] = (short)j;
          Stack stack = new Stack();
          stack.push( "S0" );
          Vector begin = new Vector( 2 );
          begin.addElement( stateTrans[0][0] );
          begin.addElement( stateTrans[0][1] );
          stack.push( begin );
          char[] partial = new char[ length ];
          int curPos = 1;
          int exitCode = 0;
          while( exitCode == 0 ){
              if( curPos == 0 ){
                  exitCode = -1;
                  System.out.println( "string creation error in AGLtask" );
                  System.exit( 1 );
              }
              Vector top = (Vector)stack.peek();
              if( top.isEmpty() ){ // this is dead path, needs replace the last transfer.
                  stack.pop();
                  curPos--;
                 ((Vector)stack.peek()).removeElementAt( 0 );
                 continue;
              }
              if( curPos == filledLocs[1] + 1 ){ // reach the second filled letter( the end )
                  char letter = ((char[])top.elementAt(0))[0];
                  char stateLetter = ((char[])top.elementAt(0))[1];
                  if( letter==testStrings[i].charAt(filledLocs[1]) && stateLetter=='8'){
                      partial[ curPos-1 ] = letter;
                      dataStrings[i] = String.valueOf( partial );
                      exitCode = 1;
                      continue;
                  }
                  else
                      top.removeElementAt(0);
              }
              else{
                  if( curPos == filledLocs[0] + 1 ){ // reach the first filled letter.
                      int idx = 0;
                      while( idx < top.size() ){
                         char letter = ((char[])top.elementAt(idx))[0];
                         // if mismatched, remove it.
                         if( letter != testStrings[i].charAt( filledLocs[0] ) )
                             top.removeElementAt( idx );
                         else // if matched, record maximum partial string currently available.
                             idx++;
                      }
                      if( top.isEmpty() ) // no match exists.
                          continue;
                  }
                  char stateLetter = ((char[])top.elementAt(0))[1];
                  int stateIdx = Integer.parseInt(String.valueOf(stateLetter));
                  if( stateIdx < 8 ){
                      partial[ curPos-1 ] = ((char[])top.elementAt(0))[0];
                      int stateLength = stateTrans[stateIdx].length;
                      Vector elem = new Vector( stateLength );
                      for( int j = 0; j < stateLength; j++ )
                           elem.addElement( stateTrans[stateIdx][j] );
                      stack.push( elem );
                      curPos++;
                  }
                  else
                      top.removeElementAt(0);
              }
          }//end of while
          //System.out.println( dataStrings[i] );
      }//end of for
  }

  /** init CLARION options. */
  private void initClarion()
  {
/*********************** FILL IN ALL OF THE CLAIRON OPTIONS *******************/
     globals[0].setNacsOn(true);
     globals[0].setSupervisorySubsysOn(false);
     globals[0].setRtOn(true);
 /*--------------------- FEATURES INFORMATION --------------------------------*/
     //globals[0].setFeatureTypeNum( featureTypeNum );
     for( int i = 0; i < 2; i++ ){
          globals[0].setFeatureNum( i+1, featureNums[i] );
          globals[0].setFeatureLengths( i+1, featureLengths[i] );
          globals[0].setFeatureNames( i+1, featureNames[i] );
          globals[0].setFeatureValueNames( i+1, featureValueNames[i] );
     }
 /************************** SETTINGS ABOUT IDN *******************************/
 /*--------------------- Action-Centered Subsystem ---------------------------*/
     globals[0].setGsOption( gsOption );
     globals[0].setWmOption( wmOn );
     globals[0].setAcsNetNum( acsNetNum );
     globals[0].setExNetNum( exNetNum );
 /************************** SETTINGS ABOUT IDN *******************************/
     globals[0].setIdnInputDimVals( idnInputDimVals );
     globals[0].setIdnOutputDimVals( idnOutputDimVals );
     globals[0].setIdnInputFeatureInfo( idnInputFeatureInfo );
     globals[0].setIdnOutputFeatureInfo( idnOutputFeatureInfo );
     globals[0].setIdnHiddenNums( idnHiddenNums );
     globals[0].setIdnLearningOptions( idnBLLearningOptions );
     globals[0].setIdnBPLearningRates( idnBPLRs );
     globals[0].setIdnQDiscountRates( idnQDiscountRates );
     globals[0].setIdnBPMomentums( idnBPMoms );
     globals[0].setIdnActionSelectionTemps( idnActionSelTemps );
     globals[0].setOverallActionSelectionTemps( overallActionSelTemps );
 /************************** SETTINGS ABOUT ARS *******************************/
     //globals[0].setARInputNums( arInputNums );
     //globals[0].setAROutputNums( arOutputNums );
     globals[0].setARInputDimVals( arInputDimVals );
     globals[0].setAROutputDimVals( arOutputDimVals );
     globals[0].setARActionSelectionTemps( arActionSelTemps );
     globals[0].setARReasoningMethods( arReasoningMethods );
 /********************* SETTINGS ABOUT RULE LEARNING **************************/
     globals[0].setAcsKnowledgeNum( acsKnowledgeNum );
     globals[0].setRerOns( rerOns );
     globals[0].setIrlOns( irlOns );
     globals[0].setFrOns( frOns );
     globals[0].setFixRuleNums( frNums );
     globals[0].setFrActionVariedOns( frActionVariedOns );
     globals[0].setFixRuleConditions( frConditions );
     globals[0].setFixRuleActions( frActions );
 /******************* SETTINGS ABOUT RULE REPRESENTATIN ***********************/
     globals[0].setRuleUtilityOns( ruleUtilityOns );
     globals[0].setRulePartMatchOns( partMatchOns );
     globals[0].setRuleBlaOns( ruleBlaOns );
     globals[0].setActionChunkBlaOns( actionChunkBlaOns );
     globals[0].setActionRuleBlaOption( actionRuleBlaOptions );
     globals[0].setActionRuleBlaParas( actionRuleBlaParas );
     globals[0].setAcsChunkBlaOption( acsChunkBlaOptions );
     globals[0].setAcsChunkBlaParas( acsChunkBlaParas );
 /***************** SETTINGS ABOUT COORDINATING ARS AND IDN *******************/
     globals[0].setCombineDeterMode( combineDeterMode );
     globals[0].setAcsCrossCombineModes( crossCombModes );
     globals[0].setAcsCrossCombineParas( crossCombineParas );
 /************** SETTINGS ABOUT NON-ACTION-CENTERED SUBSYSTEM *****************/
 /******************** SETTINGS ABOUT NACS COMPONENTS *************************/
     globals[0].setGKSon( gksOnFlag );
     globals[0].setEMon( emOnFlag );
     globals[0].setAEMon( aemOnFlag );
    /******************** SETTINGS ABOUT GKS REPRESENATIONS **********************/
     globals[0].setGksMaxChunkNum( gksMaxChunkNum );
     // chunk selection temperature.
     globals[0].setGksOutputTemperature( gksCSTemp );
     globals[0].setESChunkOn( esChunkOnFlag );
     /******************* the following for NACS rules ********************/
     globals[0].setSimilarityOn( similarityOn );
     globals[0].setAssocBlaOn( assocBlaOn );
     globals[0].setNacsChunkBlaOn( nacsChunkBlaOn );
     globals[0].setAssocConditionWeightMode( assocConditionWeightMode );
     globals[0].setAssocDimensionWeightMode( assocDimensionWeightMode );
     globals[0].setAssocBlaOption( assocBlaOption );
     globals[0].setAssocBla( assocBla );
     //globals[0].setAssocBlaParas( assocBlaParas );
     globals[0].setChunkBlaOption( chunkBlaOption );
     globals[0].setChunkBla( chunkBla );
     //globals[0].setChunkBlaParas( chunkBlaParas );
     globals[0].setEmBlaOptions( emBlaOptions );
     globals[0].setEmBlas( emBlas );
     globals[0].setEmBlaParas( emBlaParas );
     globals[0].setThreshold_EM( threshold_EM );
     globals[0].setEmCapacity( emCapacity );
     globals[0].setEmTrainAcsProb( emTrainProb );
     globals[0].setEmTrainAcsItemNum( c9 );
     globals[0].setEmTrainAcsIterNum( c10 );
    /*********************** SETTINGS ABOUT GKS LEARNINGS ************************/
     globals[0].setAssocExtractOn( assocExtractOn );
     globals[0].setAssocEncodeExternOn( assocEncodeExternOn );
     globals[0].setChunkEncodeExternOn( chunkEncodeExternOn );
     globals[0].setChunkFromAmnOn( chunkFromAmnOn );
     globals[0].setChunkEncodeProb( p_c );
     globals[0].setAssocEncodeProb( p_a );
     globals[0].setChunkDensity( d_c );
     globals[0].setAssocDensity( d_a );
     globals[0].setAssocExtThreshold( threshold_ae );
     globals[0].setChunkExtThreshold( threshold_ce );
     globals[0].setAssocExtProb( p_ae );
     globals[0].setChunkExtProb( p_ce );
    /******************** SETTINGS ABOUT AMN REPRESENATIONS **********************/
    /*-------- AMN SETTINGS --------*/
     globals[0].setAmnNum( amnNum );
    // include each feature's type and order in that type.
     globals[0].setAmnInputFeatureInfo( amnInputFeatureInfo );
     // include each feature's type and order in that type.
     globals[0].setAmnOutputFeatureInfo( amnOutputFeatureInfo );
     globals[0].setAmnInputDimVals( amnInputDimVals );
     globals[0].setAmnOutputDimVals( amnOutputDimVals );
     globals[0].setAmnHiddenNums( amnHiddenNums );
     globals[0].setAmnMultiPassOns( amnMultiPassOns );
     globals[0].setAmnPassNums( amnIterNums );
    /*-------- AMN TRAINING SETTINGS --------*/
     globals[0].setAmnTrainedByEmProb( amnTrainProb );
     globals[0].setAmnTrainedByEmItemNum( amnTrainItemNum );
     globals[0].setAmnTrainedByEmIterNum( amnTrainIterNum );
    /*-------- AMN BP SETTINGS --------*/
     globals[0].setAmnBPLearningRates( amnLRs );
     globals[0].setAmnBPMomentums( amnMoms );
     /*-------- AEM SETTINGS --------*/
     globals[0].setAemStateNumBound( aemStateNumBound );
     globals[0].setAemTrainedByEmProb( aemTrainedByEmProb );
     globals[0].setAemTrainedByEmItemNum( aemTrainedByEmItemNum );
     globals[0].setAemTrainedByEmBPLearningRate( aemTrainedByEmLR );
     globals[0].setAemTrainedByEmBPMomentum( aemTrainedByEmMom );
     globals[0].setAemPotentialNextStateNum( aemNSNum );
     globals[0].setAemTrainAcsProb( aemTrainAcsProb );
     globals[0].setAemTrainAcsSampleNum( aemTrainAcsSampleNum );
     globals[0].setAemTrainAcsIterNum( aemTrainAcsIterNum );
     globals[0].setAemTrainAcsOutputTemp( aemTrainAcsOutputTemp );
 /*************** SETTINGS ABOUT RESPONSE TIME DETERMINATION ******************/
     globals[0].setRTOn( rtOn );
     //globals[0].setMaxUsageNum( maxUsageNum );
     globals[0].setPT_BL( PT_BL );
     globals[0].setDT_BL( DT_BL );
     globals[0].setAT_BL( AT_BL );
     globals[0].setPT_TL( PT_TL );
    // used for calculating DT_TL = DT_TL_actioTime + t0/rule_bla + t1/chunk_bla;
     globals[0].setDT_TL_actionTime(  DT_TL_actionTime );
     globals[0].setDT_TL_t0( t0 );
     globals[0].setDT_TL_t1( t1 );
     globals[0].setAT_TL( AT_TL );
     globals[0].setAssocAppTime_t4( t4 );
     globals[0].setAssocAppTime_t5( t5 );
     globals[0].setChunkRetTime_t2( t2 );
     globals[0].setChunkRetTime_t3( t3 );
     globals[0].setAmnOnePassTime( t_AMN );
 /************************ SETTING ABOUT TASK-SPECIFIC ************************/
     globals[0].setTaskName( taskName );
     globals[0].setRootDir( rootPath );
     globals[0].setSettingDir( optionFileRelativePath );
     globals[0].setDataDir( dataFileRelativePath );
     globals[0].setInputFormat( inputFormat );
     globals[0].setOutputFormat( outputFormat );
  }

  private void gotoEndThisAgent()
  {
     saveAgentData();
     if( agentIdx + 1 < AGENT_NUM ){
         agentIdx++;
         initNewAgent();
     }
     else{
         //saveGroupData();
         if( groupIdx + 1 < GROUP_NUM ){
             groupIdx++;
             initNewGroup();
         }
         else{
             saveAverageModelData();
             saveWholeModelData();
             finishFlag = true;
         }
     }
  }

  private void initNewAgent()
  {
     if( (agentIdx % 1 ) == 0 )
         System.out.println( "cur agent no = " + agentIdx );
     if( agents[0] == null ){
         globals[0].setAgentGroupId( groupIdx );
         agents[0] = new AgentAGL( globals[0], trainSet, testSet );
     }
     else{
         globals[0].setAgentGroupId( groupIdx );
         agents[0].reinit();
     }
  }

  private void initNewGroup()
  {
     System.out.println( "cur group no = " + groupIdx );
     agentIdx = 0;
     initNewAgent();
     ((AgentAGL)agents[0]).setGroupType( groupTypes[ groupIdx ] );
  }

/**************************** SAVE DATA METHODS *******************************/
  private void saveAgentData()
  {
     String str = "Current agent = " + String.valueOf( agentIdx ) + "\n";
     for( int i = 0; i < SESSION_NUM; i++ ){
          str += "Current session = " + String.valueOf(i+1) + "\n";
          str += "The achievement = " + wholeModelData[groupIdx][agentIdx][0][i] + "\n";
          str += "The accuracy = " + wholeModelData[groupIdx][agentIdx][1][i] + "\n";
          str += "The efficiency = " + wholeModelData[groupIdx][agentIdx][2][i] + "\n";
          str += "The speed = " + wholeModelData[groupIdx][agentIdx][3][i] + "\n";
          str += "The number of attempts = " + wholeModelData[groupIdx][agentIdx][4][i] + "\n";
     }
     agentDataFiles[groupIdx].saveStringData( str );
  }

  private void saveAverageModelData()
  {
     String str = "";
     for( int i = 0; i < MEASURE_NUM; i++ )
        for( int j = 0; j < GROUP_NUM; j++ ){
             str += "MEASURE = " + measureNames[i] + ", GROUP = " + this.newGroupNames[j] + " or " +
                    this.oldGroupNames[j] + "\n";
             for( int k = 0; k < SESSION_NUM; k++ ){
                  double sum = 0.0;
                  for( int m = 0; m < AGENT_NUM; m++ )
                       sum += wholeModelData[j][m][i][k];
                  sum /= AGENT_NUM;
                  str += String.valueOf( (double)(Math.round( sum*100 )) / 100 );
                  str += "\n";
             }
             str += "\n";
     }
     modelDataFile.saveStringData( str );
  }

  private void saveWholeModelData()
  {
      for( int i = 0; i < SESSION_NUM; i++ )
       for( int j = 0; j < MEASURE_NUM; j++ ){
            String str = "";
            for( int k = 0; k < GROUP_NUM; k++ ){
                 str += "GROUP = " +  newGroupNames[k] + " or " + oldGroupNames[k] + "\n";
                 for( int m = 0; m < AGENT_NUM; m++ )
                      str += String.valueOf( (double)(Math.round( wholeModelData[k][m][j][i]*100 )) / 100 ) + "\n";
                 str += "\n";
            }
            wholeModelDataFiles[i][j].saveStringData( str );
       }
   }
 }
